%% Function to run simulation and make plots
clear all;
close all;
clc;

%Get constants
plantPar    = getPlantPar();
tuningPar   = getTuningPar();
initPar     = getInitPar(plantPar, tuningPar);

[qpump qback hbit VaDot] = makeCase();
%qpump = max(0, qpump);
%qpump = max(0,qpump);

%Run simulation
[Pp, Pc, qbit, qchoke, Pbit, qbitHat, PbitHat, thetaHat, PfricA, PfricAHat, Zc]...
    = simulateKaasaCrazy(plantPar, tuningPar, initPar, qpump, qback);



%PLOTS
time = 0:tuningPar.simLength;
M = plantPar.Ma + plantPar.Md;
g = plantPar.g;
rhod = plantPar.rhod;
rhoa = plantPar.rhoa;
Fd = plantPar.Fd;
Fa = plantPar.Fa;
F = Fd + Fa;
theta = tuningPar.theta;
%%
figure
plot(1:length(qpump), qpump*6e4, 1:length(qpump), qback*6e4);
axis([0 length(qpump), -100 2100]);
xlabel('Time [s]');
ylabel('Flow [ltr/min]');
l=legend('q_{pump}', 'q_{back}','Location','East');
set(l,'FontSize',8)
grid on
set(gcf, 'PaperUnits', 'centimeters');
set(gcf, 'PaperSize', [10 10]);
set(gcf, 'PaperPosition', [0 0 10 10]);
%pause
%saveas(gcf,'C:\Master\LaTeX\fig\fricCaseAddSimqpump','pdf')
%%
figure();
plot(time, Pp,time, 10*Pc, time, Zc*100);
ylabel('Pressure [Bar]');
xlabel('Time [s]');
legend('Pp', '10*Pc', '100*Zc');
grid on;
set(gcf, 'PaperUnits', 'centimeters');
set(gcf, 'PaperSize', [15 4]);
set(gcf, 'PaperPosition', [0 0 15 4]);
%pause
%saveas(gcf,'C:\Master\LaTeX\fig\fricCaseKaasaPp','pdf')
%%
figure
plot(time, Pbit, time, PbitHat,'r--');
xlabel('Time [s]');
ylabel('Pressure [Bar]');
legend('P_{bit}','P_{bit}Hat');
axis([0 length(time) 220 250])
grid on;
set(gcf, 'PaperUnits', 'centimeters');
set(gcf, 'PaperSize', [15 4]);
set(gcf, 'PaperPosition', [0 0 15 4]);
%pause
%saveas(gcf,'C:\Master\LaTeX\fig\fricCaseKaasaPbit','pdf')


%% Estimation

figure();
plot(time, qbit*6e4, time, qbitHat*6e4,'r--');
xlabel('Time [s]');
ylabel('Flow [ltr/min]');
l =legend('q_{bit}','q_{bit}Hat','location','east');
set(l,'FontSize',8);
grid on;
set(gcf, 'PaperUnits', 'centimeters');
set(gcf, 'PaperSize', [15 4]);
set(gcf, 'PaperPosition', [0 0 15 4]);
%pause
%saveas(gcf,'C:\Master\LaTeX\fig\fricCaseqbit','pdf')
%%
    figure
    subplot(411), plot(time, theta(1)*ones(1,tuningPar.simLength+1), time, thetaHat(1,:),'r--');
    xlabel('Time [s]');
    ylabel('[]');
    axis([0 4500 0 5]);
    l = legend('\theta_1','\theta_1 Hat');
    set(l,'FontSize', 8, 'Orientation', 'Horizontal', 'Location','SouthEast');
    grid on;

    subplot(412), plot(time, theta(2)*ones(1,tuningPar.simLength+1), time, thetaHat(2,:),'r--');
    xlabel('Time [s]');
    ylabel('[]');
    axis([0 4500 0 8]);
    l=legend('\theta_2','\theta_2 Hat');
    set(l,'FontSize', 8, 'Orientation', 'Horizontal', 'Location','SouthEast');
    grid on;

    subplot(413), plot(time, theta(3)*ones(1,tuningPar.simLength+1), time, thetaHat(3,:),'r--');
    xlabel('Time [s]');
    ylabel('[]');
    axis([0 4500 0 11]);
    l=legend('\theta_3','\theta_3 Hat');
    set(l,'FontSize', 8, 'Orientation', 'Horizontal', 'Location','SouthEast');
    grid on;
    
    subplot(414), plot(time, theta(4)*ones(1,tuningPar.simLength+1), time, thetaHat(4,:),'r--');
    xlabel('Time [s]');
    ylabel('[]');
    l=legend('\theta_4','\theta_4 Hat');
    set(l,'FontSize', 8, 'Orientation', 'Horizontal', 'Location','SouthEast');
    grid on;
    axis([0 4500 0 20]);
    set(gcf, 'PaperUnits', 'centimeters');
    set(gcf, 'PaperSize', [15 15]);
    set(gcf, 'PaperPosition', [0 0 15 15]);
   % pause
   % saveas(gcf,'C:\Master\LaTeX\fig\fricCaseKaasaThetaAll','pdf')

%% Error
% 
% figure();
% subplot(311), plot(time, qbit-qbitHat);
% xlabel('Time [s]');
% ylabel('Value [?]');
% legend('qbit-qbitHat');
% grid on;
% 
% 
% subplot(312), plot(time, (Fa+Fd)/M*ones(1,tuningPar.simLength+1)-(thetaHat(1,:)));
% xlabel('Time [s]');
% ylabel('Value [?]');
% legend('F-FHat');
% grid on;
% subplot(313), plot(time, 0*ones(1,tuningPar.simLength+1)-(thetaHat(2,:)));
% xlabel('Time [s]');
% ylabel('Value [?]');
% legend('rho-rhoHat');
% grid on;

%% Friction
figure

plot(time, PfricA, time, PfricAHat,'r--', time, PfricA-PfricAHat,'-.');
xlabel('Time [s]');
ylabel('Pressure [bar]');
grid on;
l=legend('P_{fric}^a', 'P_{fric}^a Hat', 'Error');
set(l, 'FontSize',8,'location','southeast');
grid on;
set(gcf, 'PaperUnits', 'centimeters');
set(gcf, 'PaperSize', [15 7]);
set(gcf, 'PaperPosition', [0 0 15 7]);
%pause
%saveas(gcf,'C:\Master\LaTeX\fig\fricCaseKaasaPfricError','pdf')



%% 3D plot
% clear Pf3d
% Pf3dHat = zeros(201,101);
% Pf3d    = zeros(201,101);
% phi = getBasisFunc(tuningPar.basisNumber);
% 
% i = 1;
% j = 1;
% for t = 0:20:tuningPar.simLength
%     j = 1;
%     for q = 0:20:2000
%         Pf3dHat(i,j) = getFriction(q/6e4,thetaHat(:,t+1), phi, tuningPar.basisNumber);
%         Pf3d(i,j) = getFriction(q/6e4,theta, phi, tuningPar.basisNumber);
%         j = j+1;
%     end
%     i = i+1;
% end
% 
% figure
% %surf(1:20:2000,1:20:4000,Pf3d(2:end,2:end)-Pf3dHat(2:end,2:end));
% surf(1:20:2000,1:20:4000,Pf3dHat(2:end,2:end));
% 
% xlabel('Flow');
% ylabel('Time');
% zlabel('P_{fric}');


        
  
%% 3D plot 
clear Pf3dHat
clc
%Pf3dHat = zeros(201,101);
%Pf3d    = zeros(201,101);
g = 1;
qpSpan = 1:2000;
c = [0 500 1000 1500 2000 3000]; 

for j = qpSpan
        phi.phi1(g) = calcPhi(j,1,1,c);
        phi.phi2(g) = calcPhi(j,1,2,c);
        phi.phi3(g) = calcPhi(j,1,3,c);
        phi.phi4(g) = calcPhi(j,1,4,c);

        g = g+1;
end

step = 20;
tstep = 60;

i = 1;
j = 1;
for t = 0:tstep:tuningPar.simLength-1
    j = 1;
    for q = 0:step:2000
        Pf3dHat(i,j) = getFriction(q/6e4,thetaHat(:,t+1), phi, tuningPar.basisNumber);
     %   Pf3d(i,j) = getFriction(q/6e4,theta, phi, 5);
        j = j+1;
    end
    i = i+1;
end

figure
%surf(1:20:2000,1:20:4000,Pf3d(2:end,2:end)-Pf3dHat(2:end,2:end));
surf(0:step:2000,0:tstep:tuningPar.simLength-1,Pf3dHat(1:end,1:end));

xlabel('Flow [ltr/min]');
ylabel('Time [s]');
zlabel('P_{fric} [Bar]');
axis([0 2000 0 4500 0 16])
set(gcf, 'PaperUnits', 'centimeters');
set(gcf, 'PaperSize', [15 10]);
set(gcf, 'PaperPosition', [0 0 15 10]);
%pause
%saveas(gcf,'C:\Master\LaTeX\fig\fricCaseKaasa3D','pdf');
%close all




