function [Pp, Pc, qbit, qchoke, Pbit, qbitHat, PbitHat, thetaHat, PfricA, PfricAHat,Zc] =...
    simulateKaasaCrazy(plantPar, tuningPar, initPar, qpump, qback)


y0 = [initPar.Pp0 initPar.Pc0 initPar.qbit0 initPar.xiHat0 ...
      initPar.thetaHat0]';

simLength   = tuningPar.simLength;
n           = tuningPar.basisNumber;
Pp      = zeros(1, simLength+1);
Pc      = zeros(1, simLength+1);
Pbit    = zeros(1, simLength+1);
PbitHat = zeros(1, simLength+1);
qbit    = zeros(1, simLength+1);
qbitHat = zeros(1, simLength+1);
qchoke  = zeros(1, simLength+1);
thetaHat= zeros(n, simLength+1);
PfricA  = zeros(1, simLength+1);
PfricAHat= zeros(1, simLength+1);
Zc      = zeros(1, simLength+1);

Pp(1)       = initPar.Pp0;
Pc(1)       = initPar.Pc0;
qbit(1)     = initPar.qbit0;
qbitHat(1)  = initPar.qbitHat0;
thetaHat(:,1) = initPar.thetaHat0;
Pbit(1)     = 220;
PbitHat(1)  = 200;

  
simLength   = tuningPar.simLength;
stepSize    = tuningPar.stepSize;

%phi = getBasisFunc(n);
g = 1;
qpSpan = 1:2000;
c = [0 500 1000 1500 2000 3000]; 

for j = qpSpan
        phi.phi1(g) = calcPhi(j,1,1,c);
        phi.phi2(g) = calcPhi(j,1,2,c);
        phi.phi3(g) = calcPhi(j,1,3,c);
        phi.phi4(g) = calcPhi(j,1,4,c);
        g = g+1;
end
qbitltr0    = initPar.qbit0*6e4; 
PfricA(1)   = max(0,2.1e-9*qbitltr0^3 - 4.7e-6*qbitltr0^2 + 8.9e-3*qbitltr0-0.21);
PfricAHat(1)= getFriction(initPar.qbitHat0, initPar.thetaHat0, phi,n);

for time = 2:simLength+1
    timeSpan = time:stepSize:time+1;
 
    [yOut xtraOut] = ode4m(@(t, y) totalSystem(t, y, plantPar, tuningPar, qpump(mod(time,1601)+1), qback(mod(time,1601)+1), Pp(time-1), phi,n), timeSpan, y0);
    
    Pp(time)        = max(1,yOut(end, 1));
    Pc(time)        = max(0,yOut(end, 2));
    qbit(time)      = max(0,yOut(end, 3));
 %  xiHat(time)     = yOut(end, 4);
    thetaHat(:,time)= yOut(end,5:end);
    
    qchoke(time)    = xtraOut(end).var1;
    Pbit(time)      = xtraOut(end).var2;
    PbitHat(time)   = xtraOut(end).var3;
    qbitHat(time)   = xtraOut(end).var4;
    Zc(time)        = xtraOut(end).var5;
    PfricA(time)    = xtraOut(end).var6;
    PfricAHat(time) = xtraOut(end).var7;
    
    y0 = [max(1,yOut(end,1)) max(0,yOut(end,2)) max(0,yOut(end,3)) yOut(end,4) yOut(end,5:end)];

end   
end

function [dydt xtra] = totalSystem(t,y,plantPar, tuningPar, qpump, qback, PpLast, phi,n)
persistent eSum;
if(isempty(eSum))
    eSum = 0;
end
%Plant parameters
Vd      = plantPar.Vd;
betad   = plantPar.betad;
Va      = plantPar.Va;
betaa   = plantPar.betaa;
Ma      = plantPar.Ma;
Md      = plantPar.Md;
M       = Ma+Md;
Fd      = plantPar.Fd;
Fa      = plantPar.Fa;

rhod    = plantPar.rhod;
rhoa    = plantPar.rhoa;
Kc      = plantPar.Kc;
P0      = plantPar.p0;
g       = plantPar.g;
hbit    = plantPar.hbit;


qres    = 0;

l1      = tuningPar.l1;

%Some defined constants
VaDot = 0;
a1 = betad/Vd;
a2 = 1/M;

%States
Pp      = max(1,y(1));
Pc      = max(0,y(2));
qbit    = max(0,y(3));
xiHat   = y(4);


thetaHat=y(5:end);

%Use curvefit function as real friction i annulus.
qbitLtr = qbit*6e4;
PfricA  = max(0,2.1e-9*qbitLtr^3 - 4.7e-6*qbitLtr^2 + 8.9e-3*qbitLtr-0.21);

% Choke
PcRef   = tuningPar.PcRef;
e       = Pc - PcRef;
eSum    = eSum + e;
Kp      = tuningPar.Kp;
Ki      = tuningPar.Ki;
Zc      = Kp*e + Ki*eSum;
Zc      = max(0,Zc);
Zc      = min(Zc, 1);
qchoke  = Kc*Zc*sqrt(2/rhoa*(Pc-P0));

% Plant

PpDot   = a1*(qpump - qbit);
PcDot   = betaa/Va*(qbit + qback - qchoke + qres - VaDot);
qbitDot = a2*(Pp - Pc -Fd*abs(qbit)*qbit -...
            PfricA + (rhod - rhoa)*g*hbit);

Pbit    = Pc + Ma*qbitDot + PfricA + rhoa*g*hbit;

%Observer
qbitHat = xiHat - l1*Pp;

PfricAHat = getFriction(qbitHat, thetaHat, phi,n);


if (qbitHat <= 0 && ((Pp-Pc) < 0))
    xiHatDot = l1*a1*qpump;
else
xiHatDot= -l1*a1*(qbitHat-qpump) - a2*Fd*abs(qbitHat)*qbitHat ...
        - a2*PfricAHat  + a2*(Pp-Pc);
end

 
    
PbitHat = Pc + Ma*(a2*(Pp - Pc -Fd*abs(qbitHat)*qbitHat -...
            PfricAHat)) + PfricAHat + (rhod*g-M*0)*hbit;

        
phi1    = phi.phi1;
phi2    = phi.phi2;
phi3    = phi.phi3;
phi4    = phi.phi4;

%Passive identifier (15)
qbIndex = max(1,round(qbitHat*6e4));
qbIndex = min(2000, qbIndex);

%8 9 10 40
thetaHatDot(1) = -75*phi1(qbIndex)*(qbit-qbitHat);
thetaHatDot(2) = -125*phi2(qbIndex)*(qbit-qbitHat);
thetaHatDot(3) = -50*phi3(qbIndex)*(qbit-qbitHat);
thetaHatDot(4) = -50*phi4(qbIndex)*(qbit-qbitHat);

if (qbitHat < 0)
    thetaHatDot(1:4) = 0;
end

% Additional output variables
xtra.var1 = qchoke;
xtra.var2 = Pbit;
xtra.var3 = PbitHat;
xtra.var4 = qbitHat;
xtra.var5 = Zc;
xtra.var6 = PfricA;
xtra.var7 = PfricAHat;


%Collect the dynamtics
dydt = [PpDot; PcDot; qbitDot; xiHatDot; thetaHatDot'];

end