%% Function to run simulation and make plots
clear all;
close all;
clc;

%Get constants
plantPar    = getPlantPar();
tuningPar   = getTuningPar();
initPar     = getInitPar(plantPar, tuningPar);
%[qpump qback]= getqpump(1000);

qpump(1:300) = 2000/6e4;
qpump(301:600) = 1800/6e4;
qpump(601:900) = 1600/6e4;
qpump(901:1200) = 1400/6e4;
qpump(1201:1500) = 1000/6e4;
qpump(1501:1800) = 800/6e4;
qpump(1801:2100) = 600/6e4;
qpump(2101:2300) = 300/6e4;
qpump(2301:2500) = 800/6e4;
qpump(2501:2800) = 1300/6e4;
qpump(2801:3000) = 1700/6e4;
qpump(3001:4001) = 2000/6e4;
qback = 0;

basisNumber = 5;

%Run simulation
[Pp, Pc, qbit, qchoke, Pbit, qbitHat, PbitHat, thetaHat,PfricA, PfricAHat]...
    = simulateGrip(plantPar, tuningPar, initPar, qpump, qback,basisNumber);

%PLOTS
time = 0:tuningPar.simLength;
M = plantPar.Ma + plantPar.Md;
g = plantPar.g;
rhod = plantPar.rhod;
rhoa = plantPar.rhoa;
Fd = plantPar.Fd;
Fa = plantPar.Fa;
F = Fd + Fa;
theta = tuningPar.theta5;


%% Estimation

figure();
plot(time, qbit*6e4, time, qbitHat*6e4,'r--');
xlabel('Time [s]');
ylabel('Flow [m^3/h]');
legend('qbit','qbitHat');
grid on;

figure();
subplot(511), plot(time, theta(1)*ones(1,tuningPar.simLength+1), time, thetaHat(1,:),'r--');
xlabel('Time [s]');
ylabel('[]');
legend('\theta_1','\theta_1 Hat');
grid on;

subplot(512), plot(time, theta(2)*ones(1,tuningPar.simLength+1), time, thetaHat(2,:),'r--');
xlabel('Time [s]');
ylabel('[]');
legend('\theta_2','\theta_2 Hat');
grid on;

subplot(513), plot(time, theta(3)*ones(1,tuningPar.simLength+1), time, thetaHat(3,:),'r--');
xlabel('Time [s]');
ylabel('[]');
legend('\theta_3','\theta_3 Hat');
grid on;


subplot(514), plot(time, theta(4)*ones(1,tuningPar.simLength+1), time, thetaHat(4,:),'r--');
xlabel('Time [s]');
ylabel('[]');
legend('\theta_4','\theta_4 Hat');
grid on;

subplot(515), plot(time, theta(5)*ones(1,tuningPar.simLength+1), time, thetaHat(5,:),'r--');
xlabel('Time [s]');
ylabel('[]');
legend('\theta_5','\theta_5 Hat');
grid on;
pause
%% Pressures
figure();
subplot(211),plot(time, Pp,time, 10*Pc);
ylabel('Pressure [Bar]');
xlabel('Time [s]');
legend('Pp', '10*Pc');
grid on;


subplot(212),plot(time, Pbit);
xlabel('Time [s]');
ylabel('Pressure [Bar]');
legend('Pbit');
grid on;
%%
%Flows
figure();
subplot(311), plot(time, qpump(1:tuningPar.simLength+1)*6e4, time, qback*6e4);
xlabel('Time [s]');
ylabel('Flow [l/min]');
legend('qpump');%, 'qback');
grid on;

subplot(312), plot(time, qbit*60e3, time, qbitHat*6e4,'r--');
xlabel('Time [s]');
ylabel('Flow [l/min]');
legend('qbit', 'qbit (x(2))');
grid on;

subplot(313), plot(time, qchoke*60e3);
xlabel('Time [s]');
ylabel('Flow [l/min]');
legend('qchoke');
grid on;
%% Error

figure();
subplot(311), plot(time, qbit-qbitHat);
xlabel('Time [s]');
ylabel('Value [?]');
legend('qbit-qbitHat');
grid on;


subplot(312), plot(time, (Fa+Fd)/M*ones(1,tuningPar.simLength+1)-(thetaHat(1,:)));
xlabel('Time [s]');
ylabel('Value [?]');
legend('F-FHat');
grid on;
subplot(313), plot(time, 0*ones(1,tuningPar.simLength+1)-(thetaHat(2,:)));
xlabel('Time [s]');
ylabel('Value [?]');
legend('rho-rhoHat');
grid on;

%% STates
figure();
subplot(311),plot(time, Pp,time, 10*Pc);
ylabel('Pressure [Bar]');
xlabel('Time [s]');
legend('Pp', '10*Pc');
grid on;

subplot(312),plot(time, Pbit);
xlabel('Time [s]');
ylabel('Pressure [Bar]');
legend('Pbit');
axis([0 tuningPar.simLength 200 250]);
grid on;

subplot(313), plot(time, qpump(1:tuningPar.simLength+1)*6e4, time, qback*ones(1,tuningPar.simLength+1)*6e4,time, qbit*60e3);
xlabel('Time [s]');
ylabel('Flow [l/min]');
legend('qpump', 'qback', 'qbit');
grid on;



