function [Pp, Pc, qbit, qchoke, Pbit, qbitHat, PbitHat, thetaHat, PfricA, PfricAHat] =...
    simulateGrip(plantPar, tuningPar, initPar, qpump, qback, basisNumber)


y0 = [initPar.Pp0 initPar.Pc0 initPar.qbit0 ...
    initPar.thetaHat0 initPar.qbitHat0]';

simLength = tuningPar.simLength;

Pp      = zeros(1, simLength+1);
Pc      = zeros(1, simLength+1);
Pbit    = zeros(1, simLength+1);
PbitHat = zeros(1, simLength+1);
qbit    = zeros(1, simLength+1);
qchoke  = zeros(1, simLength+1);
qbitHat = zeros(1, simLength+1);
thetaHat= zeros(5, simLength+1);
PfricA  = zeros(1, simLength+1);
PfricAHat= zeros(1, simLength+1);
Zc      = zeros(1, simLength+1);


Pp(1)       = initPar.Pp0;
Pc(1)       = initPar.Pc0;
qbit(1)     = initPar.qbit0;
qbitHat(1)  = initPar.qbitHat0;
thetaHat(:,1) = initPar.thetaHat0;


simLength   = tuningPar.simLength;
stepSize    = tuningPar.stepSize;

[phi1, phi2, phi3, phi4, phi5] = getBasisFunc(basisNumber);
phi.phi1 = phi1;
phi.phi2 = phi2;
phi.phi3 = phi3;
if basisNumber == 5
    phi.phi4 = phi4;
    phi.phi5 = phi5;
end
tuningPar.theta = tuningPar.theta5;

PfricA(1)   = getFriction(initPar.qbit0, tuningPar.theta, phi,basisNumber);
PfricAHat(1)= getFriction(initPar.qbitHat0, initPar.thetaHat0, phi,basisNumber);

for time = 2:simLength+1
    clc
    time-1
    timeSpan = time:stepSize:time+1;
    
    [yOut xtraOut] = ode4m(@(t, y) totalSystem(t, y, plantPar, tuningPar, qpump(mod(time,4001)+1), qback, Pp(time-1), phi,basisNumber), timeSpan, y0);
    
    Pp(time)        = yOut(end, 1);
    Pc(time)        = yOut(end, 2);
    qbit(time)      = yOut(end, 3);
    
    thetaHat(:,time)= yOut(end,4:8);
    qbitHat(time) = yOut(end,9);
       
    qchoke(time)    = xtraOut(end).var1;
    Pbit(time)      = xtraOut(end).var2;
    PbitHat(time)   = xtraOut(end).var3;
    qbitHat(time)   = xtraOut(end).var4;
    Zc(time)        = xtraOut(end).var5;
    PfricA(time)    = xtraOut(end).var6;
    PfricAHat(time) = xtraOut(end).var7;
    
    y0 = [yOut(end, 1) yOut(end, 2) yOut(end, 3) yOut(end,4:8) yOut(end,9)]';

end   
end

function [dydt xtra] = totalSystem(t,y,plantPar, tuningPar, qpump, qback, PpLast, phi,basisNumber)

%Plant parameters
Vd      = plantPar.Vd;
betad   = plantPar.betad;
Va      = plantPar.Va;
betaa   = plantPar.betaa;
Ma      = plantPar.Ma;
Md      = plantPar.Md;
M       = Ma+Md;
Fd      = plantPar.Fd;
Fa      = plantPar.Fa;
F       = Fa+Fd;
rhod    = plantPar.rhod;
rhoa    = plantPar.rhoa;
Kc      = plantPar.Kc;
P0      = plantPar.p0;
g       = plantPar.g;
hbit    = plantPar.hbit;

theta = tuningPar.theta;

%Tuning parameters
qres    = 0;%tuningPar.qres;

gamma0  = tuningPar.gamma0;
Gamma   = tuningPar.GammaGBig;


%Some defined constants
VaDot = 0;
a1 = betad/Vd;
a2 = 1/M;

%States
Pp      = y(1);
Pc      = y(2);
qbit    = y(3);
thetaHat= y(4:8);
qbitHat = y(9);


% Choke
Zc      = tuningPar.Zc;
Zc      = max(0,Zc);
Zc      = min(Zc, 1);
qchoke  = Kc*Zc*sqrt(2/rhoa*(Pc-P0));

% Plant
PpDot   = a1*(qpump - qbit);
PcDot   = betaa/Va*(qbit + qback -qchoke + qres - VaDot);
qbitDot = a2*(Pp - Pc -Fd*abs(qbit)*qbit -...
            getFriction(qbit, theta, phi,basisNumber) + (rhod - rhoa)*g*hbit);

Pbit    = Pc + Ma*qbitDot + Fa*abs(qbit+qres)*(qbit+qres) + rhoa*g*hbit;

n = basisNumber;

%System
  %Make qbit from known signals
  PpDotEst  = Pp-PpLast;
  qbitKnown = qpump - 1/a1*PpDotEst;

f0      = a2*(Pp - Pc);

%Grip observer
phi1    = phi.phi1;
phi2    = phi.phi2;
phi3    = phi.phi3;
phi4    = phi.phi4;
phi5    = phi.phi5;

qbIndex = max(1,round(qbitHat*6e4));
qbIndex = min(2000, qbIndex);
%psiHat = gamma0*(qbitKnown-qbitHat) + getFriction(qbitHat, thetaHat, phi,n);

thetaHatDot(1) = -1e1*phi1(qbIndex)*gamma0*(qbitKnown-qbitHat);
thetaHatDot(2) = -1e1*phi2(qbIndex)*gamma0*(qbitKnown-qbitHat);
thetaHatDot(3) = -1e1*phi3(qbIndex)*gamma0*(qbitKnown-qbitHat);
thetaHatDot(4) = -1e1*phi4(qbIndex)*gamma0*(qbitKnown-qbitHat);
thetaHatDot(5) = -1e1*phi5(qbIndex)*gamma0*(qbitKnown-qbitHat);


qbitHatDot  = f0 + gamma0*(qbitKnown-qbitHat) - a2*Fd*qbitHat^2 - a2*getFriction(qbitHat, thetaHat, phi,n)... 
    + inv(gamma0).*[phi1(qbIndex);phi2(qbIndex);phi3(qbIndex);phi4(qbIndex);phi5(qbIndex)]'*thetaHatDot';


PbitHat = Pbit; %Pc + Ma*(a2*(Pp-Pc)-thetaHat(1)*abs(qbitHat)*qbitHat+thetaHat(2)*hbit)...
 %           + (M*thetaHat(1)-Fd)*abs(qbitHat)*qbitHat + (rhod*g - M*thetaHat(2))*hbit;

%Additional output variables
xtra.var1 = qchoke;
xtra.var2 = Pbit;
xtra.var3 = PbitHat;
xtra.var4 = qbitHat;
xtra.var5 = Zc;
xtra.var6 = getFriction(qbit,theta,phi,basisNumber);
xtra.var7 = getFriction(qbitHat,thetaHat,phi,basisNumber);

%Collect the dynamtics
dydt = [PpDot; PcDot; qbitDot; thetaHatDot(1); thetaHatDot(2); thetaHatDot(3); thetaHatDot(4); thetaHatDot(5); qbitHatDot];

end