function [Pp, Pc, qbit, qchoke, Pbit, qbitHat, PbitHat, thetaHat, PfricA, PfricAHat] =...
    simulateKaasa(plantPar, tuningPar, initPar, qpump, qback, basisNumber)


y0 = [initPar.Pp0 initPar.Pc0 initPar.qbit0 initPar.xiHat0 ...
      initPar.thetaHat0]';

simLength = tuningPar.simLength;

Pp      = zeros(1, simLength+1);
Pc      = zeros(1, simLength+1);
Pbit    = zeros(1, simLength+1);
PbitHat = zeros(1, simLength+1);
qbit    = zeros(1, simLength+1);
qbitHat = zeros(1, simLength+1);
qchoke  = zeros(1, simLength+1);
thetaHat= zeros(5, simLength+1);
PfricA  = zeros(1, simLength+1);
PfricAHat= zeros(1, simLength+1);
Zc      = zeros(1, simLength+1);

Pp(1)       = initPar.Pp0;
Pc(1)       = initPar.Pc0;
qbit(1)     = initPar.qbit0;
qbitHat(1)  = initPar.qbitHat0;
thetaHat(:,1) = initPar.thetaHat0;


simLength   = tuningPar.simLength;
stepSize    = tuningPar.stepSize;

phi = getBasisFunc(basisNumber);


PfricA(1)   = getFriction(initPar.qbit0, tuningPar.theta, phi,basisNumber);
PfricAHat(1)= getFriction(initPar.qbitHat0, initPar.thetaHat0, phi,basisNumber);

for time = 2:simLength+1
    timeSpan = time:stepSize:time+1;
   
    [yOut xtraOut] = ode4m(@(t, y) totalSystem(t, y, plantPar, tuningPar, qpump(mod(time,240)+1), qback, Pp(time-1), phi,basisNumber), timeSpan, y0);
    
    Pp(time)        = max(0,yOut(end, 1));
    Pc(time)        = max(plantPar.p0,yOut(end, 2));
    qbit(time)      = max(0,yOut(end, 3));
 %  xiHat(time)     = yOut(end, 4);
    thetaHat(:,time)= yOut(end,5:9);
    
    qchoke(time)    = xtraOut(end).var1;
    Pbit(time)      = xtraOut(end).var2;
    PbitHat(time)   = xtraOut(end).var3;
    qbitHat(time)   = xtraOut(end).var4;
    Zc(time)        = xtraOut(end).var5;
    PfricA(time)    = xtraOut(end).var6;
    PfricAHat(time) = xtraOut(end).var7;
    
    y0 = [yOut(end, 1) yOut(end, 2) yOut(end, 3) yOut(end,4) yOut(end,5:9)];

end   
end

function [dydt xtra] = totalSystem(t,y,plantPar, tuningPar, qpump, qback, PpLast, phi,basisNumber)

%Plant parameters
Vd      = plantPar.Vd;
betad   = plantPar.betad;
Va      = plantPar.Va;
betaa   = plantPar.betaa;
Ma      = plantPar.Ma;
Md      = plantPar.Md;
M       = Ma+Md;
Fd      = plantPar.Fd;

rhod    = plantPar.rhod;
rhoa    = plantPar.rhoa;
Kc      = plantPar.Kc;
P0      = plantPar.p0;
g       = plantPar.g;
hbit    = plantPar.hbit;

%qpump   = tuningPar.qpump;
%qback   = tuningPar.qback;
qres    = 0;

l1      = tuningPar.l1;

%Some defined constants
VaDot = 0;
a1 = betad/Vd;
a2 = 1/M;

%States
Pp      = max(0,y(1));
Pc      = max(P0,y(2));
qbit    = max(0,y(3));
xiHat   = y(4);

%theta3Hat=y(6);

thetaHat=y(5:9);
theta = tuningPar.theta;
% Choke
Zc      = tuningPar.Zc;
Zc      = max(0,Zc);
Zc      = min(Zc, 1);
qchoke  = Kc*Zc*sqrt(2/rhoa*(Pc-P0));

% Plant
PpDot   = a1*(qpump - qbit);
PcDot   = betaa/Va*(qbit + qback - qchoke + qres - VaDot);
qbitDot = a2*(Pp - Pc -Fd*abs(qbit)*qbit -...
            getFriction(qbit, theta, phi,basisNumber) + (rhod - rhoa)*g*hbit);

Pbit    = Pc + Ma*qbitDot + getFriction(qbit, theta, phi,basisNumber) + rhoa*g*hbit;

%Observer
qbitHat = xiHat - l1*Pp;

xiHatDot= -l1*a1*(qbitHat-qpump) - a2*Fd*qbitHat^2 ...
        - a2*getFriction(qbitHat, thetaHat, phi,basisNumber) + a2*(rhod-rhoa)*g*hbit + a2*(Pp-Pc);

PbitHat = Pc + Ma*qbitDot + getFriction(qbitHat, thetaHat, phi,basisNumber) + rhoa*g*hbit;

        
phi1    = phi.phi1;
phi2    = phi.phi2;
phi3    = phi.phi3;
phi4    = phi.phi4;
phi5    = phi.phi5;

%Passive identifier (15)
qbIndex = max(1,round(qbitHat*6e4));
qbIndex = min(2000, qbIndex);
thetaHatDot(1) = -8e0*phi1(qbIndex)*(qbit-qbitHat);
thetaHatDot(2) = -5e1*phi2(qbIndex)*(qbit-qbitHat);
thetaHatDot(3) = -5e1*phi3(qbIndex)*(qbit-qbitHat);
thetaHatDot(4) = -5e1*phi4(qbIndex)*(qbit-qbitHat);
thetaHatDot(5) = -1e2*phi5(qbIndex)*(qbit-qbitHat);

% Additional output variables
xtra.var1 = qchoke;
xtra.var2 = Pbit;
xtra.var3 = PbitHat;
xtra.var4 = qbitHat;
xtra.var5 = Zc;
xtra.var6 = getFriction(qbit, theta, phi,basisNumber);
xtra.var7 = getFriction(qbitHat, thetaHat, phi,basisNumber);

%Collect the dynamtics
dydt = [PpDot; PcDot; qbitDot; xiHatDot; thetaHatDot(1); thetaHatDot(2); thetaHatDot(3); thetaHatDot(4); thetaHatDot(5)];

end