%% Function to run simulation and make plots
clear all;
close all;
clc;

%Get constants
plantPar    = getPlantPar();
tuningPar   = getTuningPar();
initPar     = getInitPar(plantPar, tuningPar);
%[qpump qback]= getqpump(tuningPar.simLength);

%load qpump2.mat;
%qpump = max(qpump, 100/6e4);
%[qpump qback] = getqpump();
[qpump qback hbit VaDot] = makeCase();
%qback = 0;


%Run simulation
[Pp, Pc, qbit, qchoke, Pbit, qbitHat, PbitHat, thetaHat, Zc, PfricA, PfricAHat, xiO]...
    = simulateStamnes(plantPar, tuningPar, initPar, qpump, qback, plantPar.hbit, 0);



%PLOTS
time = 0:tuningPar.simLength;
M = plantPar.Ma + plantPar.Md;
g = plantPar.g;
rhod = plantPar.rhod;
rhoa = plantPar.rhoa;
Fd = plantPar.Fd;
Fa = plantPar.Fa;
F = Fd + Fa;
theta = tuningPar.theta;
figure

%%
figure
plot(1:length(qpump), qpump*6e4,1:length(qback), qback*6e4);
axis([0 length(qpump), 0 2100]);
legend('q_{p}', 'q_{back}');
xlabel('Time [s]');
ylabel('Flow [ltr/min]');
grid on
set(gcf, 'PaperUnits', 'centimeters');
set(gcf, 'PaperSize', [5 5]);
set(gcf, 'PaperPosition', [0 0 5 5]);
%pause
%saveas(gcf,'C:\Master\LaTeX\fig\fricCaseAddSimqpump','pdf')
%%
figure();
subplot(211),plot(time, Pp,time, 10*Pc);
ylabel('Pressure [Bar]');
xlabel('Time [s]');
legend('Pp', '10*Pc');
grid on;


subplot(212),plot(time, Pbit);
xlabel('Time [s]');
ylabel('Pressure [Bar]');
legend('Pbit');
grid on;
%%
%Flows
% figure();
% subplot(311), plot(time, qpump(1:time(end)+1)*6e4);%, time, qback*6e4);
% xlabel('Time [s]');
% ylabel('Flow [l/min]');
% legend('qpump');%, 'qback');
% grid on;
% 
% subplot(312), plot(time, qbit*60e3, time, qbitHat*6e4,'r--');
% xlabel('Time [s]');
% ylabel('Flow [l/min]');
% legend('qbit', 'qbit (x(2))');
% grid on;
% 
% subplot(313), plot(time, qchoke*60e3);
% xlabel('Time [s]');
% ylabel('Flow [l/min]');
% legend('qchoke');
% grid on;

%% Estimation

figure();
plot(time, qbit*6e4, time, qbitHat*6e4,'r--');
xlabel('Time [s]');
ylabel('Flow [ltr/min]');
legend('qbit','qbitHat');
grid on;
set(gcf, 'PaperUnits', 'centimeters');
set(gcf, 'PaperSize', [15 4]);
set(gcf, 'PaperPosition', [0 0 15 4]);
%pause
%saveas(gcf,'C:\Master\LaTeX\fig\fricCaseStamnesqbit','pdf')

%%
    figure
    subplot(411), plot(time, theta(1)*ones(1,tuningPar.simLength+1), time, thetaHat(1,:),'r--');
    xlabel('Time [s]');
    ylabel('[]');
  %  axis([0 4500 0 5]);
    l = legend('\theta_1','\theta_1 Hat');
    set(l,'FontSize', 8, 'Orientation', 'Horizontal', 'location','southeast');
    grid on;

    subplot(412), plot(time, theta(2)*ones(1,tuningPar.simLength+1), time, thetaHat(2,:),'r--');
    xlabel('Time [s]');
    ylabel('[]');
    axis([0 4500 0 8]);
    l=legend('\theta_2','\theta_2 Hat');
    set(l,'FontSize', 8, 'Orientation', 'Horizontal', 'location','southeast');
    grid on;

    subplot(413), plot(time, theta(3)*ones(1,tuningPar.simLength+1), time, thetaHat(3,:),'r--');
    xlabel('Time [s]');
    ylabel('[]');
    axis([0 4500 0 11]);
    l=legend('\theta_3','\theta_3 Hat');
    set(l,'FontSize', 8, 'Orientation', 'Horizontal', 'location','southeast');
    grid on;
    
    subplot(414), plot(time, theta(4)*ones(1,tuningPar.simLength+1), time, thetaHat(4,:),'r--');
    xlabel('Time [s]');
    ylabel('[]');
    axis([0 4500 0 20]);
    l=legend('\theta_4','\theta_4 Hat');
    set(l,'FontSize', 8, 'Orientation', 'Horizontal', 'location','southeast');
    grid on;
    set(gcf, 'PaperUnits', 'centimeters');
    set(gcf, 'PaperSize', [15 15]);
    set(gcf, 'PaperPosition', [0 0 15 15]);
    %pause
    %saveas(gcf,'C:\Master\LaTeX\fig\fricCaseStamnesThetaAll','pdf')

%%
% Friction
figure

plot(time, PfricA, time, PfricAHat,'r--', time, PfricA-PfricAHat,'-.');
xlabel('Time [s]');
ylabel('Pressure [bar]');
l=legend('P_{fric}^a', 'P_{fric}^a Hat', 'Error');
set(l, 'FontSize',8,'location','southeast');
grid on;
set(gcf, 'PaperUnits', 'centimeters');
set(gcf, 'PaperSize', [15 7]);
set(gcf, 'PaperPosition', [0 0 15 7]);
%pause
%saveas(gcf,'C:\Master\LaTeX\fig\fricCaseStamnesPfricError','pdf')


%% 3D plot
% clear Pf3d
% Pf3dHat = zeros(201,101);
% Pf3d    = zeros(201,101);
% phi = getBasisFunc(tuningPar.basisNumber);
% 
% i = 1;
% j = 1;
% for t = 0:20:tuningPar.simLength
%     j = 1;
%     for q = 0:20:2000
%         Pf3dHat(i,j) = getFriction(q/6e4,thetaHat(:,t+1), phi, tuningPar.basisNumber);
%         Pf3d(i,j) = getFriction(q/6e4,theta, phi, tuningPar.basisNumber);
%         j = j+1;
%     end
%     i = i+1;
% end
% 
% figure
% %surf(1:20:2000,1:20:4000,Pf3d(2:end,2:end)-Pf3dHat(2:end,2:end));
% surf(1:20:2000,1:20:4000,Pf3dHat(2:end,2:end));
% 
% xlabel('Flow');
% ylabel('Time');
% zlabel('P_{fric}');


        
%% TO LATEX
        

    figure
    subplot(411),plot(time, theta(1)*ones(1,tuningPar.simLength+1), time, thetaHat(1,:),'r--');
    xlabel('Time [s]');
    ylabel('[]');
    legend('\theta_1','\theta_1 Hat');
    grid on;
    %set(gcf,'PaperSize',[15,4]);
    
    subplot(412),plot(time, theta(2)*ones(1,tuningPar.simLength+1), time, thetaHat(2,:),'r--');
    xlabel('Time [s]');
    ylabel('[]');
    legend('\theta_2','\theta_2 Hat');
    grid on;
    %set(gcf,'PaperSize',[15,4]);
    
    subplot(413),plot(time, theta(3)*ones(1,tuningPar.simLength+1), time, thetaHat(3,:),'r--');
    xlabel('Time [s]');
    ylabel('[]');
    legend('\theta_3','\theta_3 Hat');
    grid on;
    
    subplot(414),plot(time, theta(4)*ones(1,tuningPar.simLength+1), time, thetaHat(4,:),'r--');
    xlabel('Time [s]');
    ylabel('[]');
    legend('\theta_4','\theta_4 Hat');
    grid on;
    set(gcf, 'PaperUnits', 'centimeters');
    set(gcf, 'PaperSize', [15 20]);
    set(gcf, 'PaperPosition', [0 0 15 20]);
    %pause
    %saveas(gcf,'C:\Master\LaTeX\fig\fric4BsplineThetaAll','pdf')

    %%
    figure
    plot(time, Pbit, time, PbitHat,'r--');
    xlabel('Time [s]');
    ylabel('Pressure [Bar]');
    legend('P_{bit}','P_{bit}Hat');
    %axis([0 length(time) 220 250])
    grid on;
    set(gcf, 'PaperUnits', 'centimeters');
    set(gcf, 'PaperSize', [15 4]);
    set(gcf, 'PaperPosition', [0 0 15 4]);
%    pause
%    saveas(gcf,'C:\Master\LaTeX\fig\fricCaseStamnesPbit','pdf')
    %%
    figure
    plot(time, PfricA, time, PfricAHat,'r--', time, PfricA-PfricAHat,'-.');
    xlabel('Time [s]');
    ylabel('Pressure [bar]');
    l=legend('P_{fric}^a', 'P_{fric}^a Hat', 'Error');
    set(l, 'FontSize',8);
    grid on;
    set(gcf, 'PaperUnits', 'centimeters');
    set(gcf, 'PaperSize', [15 7]);
    set(gcf, 'PaperPosition', [0 0 15 7]);
 %    pause
 %    saveas(gcf,'C:\Master\LaTeX\fig\fric10BsplinePfricError','pdf')
%     close all

%%    
    figure
    plot(time, qbit*6e4, time, qbitHat*6e4,'r--');
    xlabel('Time [s]');
    ylabel('Flow [ltr/min]');
    l1 = legend('q_{bit}','q_{bit} Hat','Location','SouthEast');
    set(l1,'FontSize',8);
    grid on;
    set(gcf, 'PaperUnits', 'centimeters');
    set(gcf, 'PaperSize', [15 4]);
    set(gcf, 'PaperPosition', [0 0 15 4]);
 %   pause
 %   saveas(gcf,'C:\Master\LaTeX\fig\fric10Bsplineqbit','pdf')
 %   close all;
    
 
%% 3D plot 
clear Pf3dHat
clc
%Pf3dHat = zeros(201,101);
%Pf3d    = zeros(201,101);
g = 1;
qpSpan = 1:2000;
c = [0 500 1000 1500 2000 3000]; 

for j = qpSpan
        phi.phi1(g) = calcPhi(j,1,1,c);
        phi.phi2(g) = calcPhi(j,1,2,c);
        phi.phi3(g) = calcPhi(j,1,3,c);
        phi.phi4(g) = calcPhi(j,1,4,c);

        g = g+1;
end

step = 20;
tstep = 60;

i = 1;
j = 1;
for t = 0:tstep:tuningPar.simLength-1
    j = 1;
    for q = 0:step:2000
        Pf3dHat(i,j) = getFriction(q/6e4,thetaHat(1:4,t+1), phi, tuningPar.basisNumber);
     %   Pf3d(i,j) = getFriction(q/6e4,theta, phi, 5);
        j = j+1;
    end
    i = i+1;
end
%%
figure
%surf(1:20:2000,1:20:4000,Pf3d(2:end,2:end)-Pf3dHat(2:end,2:end));
surf(0:step:2000,0:tstep:tuningPar.simLength-1,Pf3dHat(1:end,1:end));

xlabel('Flow [ltr/min]');
ylabel('Time [s]');
zlabel('P_{fric} [Bar]');
axis([0 2000 0 4500 0 16])
set(gcf, 'PaperUnits', 'centimeters');
set(gcf, 'PaperSize', [15 10]);
set(gcf, 'PaperPosition', [0 0 15 10]);
%pause
%saveas(gcf,'C:\Master\LaTeX\fig\fricCaseStamnes3D','pdf');
%close all


    %% theta when support 4 Bsplines
    t1 = 1:2092;
    t2 = 1004:3503;
    t3 = 2109:4000;
    t4 = 3504:4000;
    
    figure
    hold on;
    plot(t1, thetaHat(1,t1),'b--');
    plot(t2, thetaHat(2,t2),'r--');
    plot(t3, thetaHat(3,t3),'g--');
    plot(t4, thetaHat(4,t4),'c--');
    
    l = legend('\theta_1 Hat','\theta_2 Hat','\theta_3 Hat','\theta_4 Hat', 'Location','NorthWest');
    set(l,'FontSize',8);
    plot(t1, theta(1)*ones(1,length(t1)),'b');
    plot(t2, theta(2)*ones(1,length(t2)),'r');
    plot(t3, theta(3)*ones(1,length(t3)),'g');
    plot(t4, theta(4)*ones(1,length(t4)),'c');
    
    xlabel('Time [s]');
    ylabel('[]');
    set(gcf, 'PaperUnits', 'centimeters');
    set(gcf, 'PaperSize', [15 10]);
    set(gcf, 'PaperPosition', [0 0 15 10]);
    grid on
    hold off
   % pause
   % saveas(gcf,'C:\Master\LaTeX\fig\fric4BsplineThetaSupport','pdf')
    %% theta when support 10 Bsplines
    t1 = 1:300;
    t2 = 1:603;
    t3 = 301:904;
    t4 = 604:1205;
    t5 = 905:1506;
    t6 = 1206:1808;
    t7  = 1507:2112;
    t8  = 1809:2312;
    t91 = 2113:2413;
    t92 = 3027:4001;
    t10 = 2314:4001;
    t11 = 2514:3025;

    figure
    hold on
    p(1) = plot(t1, thetaHat(1,t1),'b--');
    p(2) = plot(t2, thetaHat(2,t2),'r--');
    p(3) = plot(t3, thetaHat(3,t3),'g--');
    p(4) = plot(t4, thetaHat(4,t4),'c--');
    p(5) = plot(t5, thetaHat(5,t5),'m--');
    p(6) = plot(t6, thetaHat(6,t6),'b-.');
    p(7) = plot(t7, thetaHat(7,t7),'r-.');
    p(8) = plot(t8, thetaHat(8,t8),'g-.');
    p(9) = plot(t91, thetaHat(9,t91),'m-.');
    p(10) = plot(t10, thetaHat(10,t10),'b:');
    p(11) = plot(t11, thetaHat(11,t11),'r:');
    p(12) = plot(t92, thetaHat(9,t92),'m-.');
    grid on;
    
    
    p(13) = plot(t1, theta(1)*ones(1,length(t1)),'b');
    p(14) = plot(t2, theta(2)*ones(1,length(t2)),'r');
    p(15) = plot(t3, theta(3)*ones(1,length(t3)),'g');
    p(16) = plot(t4, theta(4)*ones(1,length(t4)),'c');
    p(17) = plot(t5, theta(5)*ones(1,length(t5)),'m');
    p(18) = plot(t6, theta(6)*ones(1,length(t6)),'b');
    p(19) = plot(t7, theta(7)*ones(1,length(t7)),'r');
    p(20) = plot(t8, theta(8)*ones(1,length(t8)),'g');
    p(21) = plot(t91, theta(9)*ones(1,length(t91)),'m');
    p(22) = plot(t10, theta(10)*ones(1,length(t10)),'b');
    p(23) = plot(t11, theta(11)*ones(1,length(t11)),'r');
    p(24) = plot(t92, theta(9)*ones(1,length(t92)),'m');
    xlabel('Time [s]');
    ylabel('[]');
    grid on;
    axis([0 4000 -1 16])

    legend(p(1:5),'\theta_1 Hat','\theta_2 Hat','\theta_3 Hat','\theta_4 Hat','\theta_5 Hat');
    ah=axes('position',get(gca,'position'),'visible','off');   
    legend(ah,p(6:11),'\theta_6 Hat','\theta_7 Hat','\theta_8 Hat','\theta_9 Hat','\theta_{10} Hat','\theta_{11} Hat');
    
    set(gcf, 'PaperUnits', 'centimeters');
    set(gcf, 'PaperSize', [15 10]);
    set(gcf, 'PaperPosition', [0 0 15 10]);
    hold off
   % pause
   % saveas(gcf,'C:\Master\LaTeX\fig\fric11BsplineThetaSupport','pdf')
        
%%
   ph=plot(rand(20,9));
     set(gca,'ylim',[-2,5]);
% the engine
% - label trace 1 and 5
    legend(ph(1),'trace one');
    ah=axes('position',get(gca,'position'),...
            'visible','off');
    legend(ah,ph(5),'trace five','location','west');

