function [Pp, Pc, qbit, qchoke, Pbit, qbitHat, PbitHat, thetaHat, PfricA, PfricAHat, qbitAproc, PpDotRett, PpDotAproc] =...
    simulateKaasa(plantPar, tuningPar, initPar, qpump, qback)



y0 = [initPar.Pp0 initPar.Pc0 initPar.qbit0 initPar.xiHat0 ...
      initPar.thetaHat0]';

simLength   = tuningPar.simLength;
n           = tuningPar.basisNumber;
Pp      = zeros(1, simLength+1);
Pc      = zeros(1, simLength+1);
Pbit    = zeros(1, simLength+1);
PbitHat = zeros(1, simLength+1);
qbit    = zeros(1, simLength+1);
qbitHat = zeros(1, simLength+1);
qchoke  = zeros(1, simLength+1);
thetaHat= zeros(n, simLength+1);
PfricA  = zeros(1, simLength+1);
PfricAHat= zeros(1, simLength+1);
Zc      = zeros(1, simLength+1);

Pp(1)       = initPar.Pp0;
Pc(1)       = initPar.Pc0;
qbit(1)     = initPar.qbit0;
qbitHat(1)  = initPar.qbitHat0;
thetaHat(:,1) = initPar.thetaHat0;


simLength   = tuningPar.simLength;
stepSize    = tuningPar.stepSize;

%phi = getBasisFunc(n);
g = 1;
qpSpan = 1:2000;
c = [0 500 1000 1500 2000 3000]; 
%c = [0 200 400 600 800 1000 1200 1400 1600 1800 2000 2200];
for j = qpSpan
        phi.phi1(g) = calcPhi(j,1,1,c);
        phi.phi2(g) = calcPhi(j,1,2,c);
        phi.phi3(g) = calcPhi(j,1,3,c);
        phi.phi4(g) = calcPhi(j,1,4,c);
%         phi.phi5(g) = calcPhi(j,1,5,c);
%         phi.phi6(g) = calcPhi(j,1,6,c);
%         phi.phi7(g) = calcPhi(j,1,7,c);
%         phi.phi8(g) = calcPhi(j,1,8,c);
%         phi.phi9(g) = calcPhi(j,1,9,c);
%         phi.phi10(g) = calcPhi(j,1,10,c);
        g = g+1;
end

PfricA(1)   = getFriction(initPar.qbit0, tuningPar.theta, phi,n);
PfricAHat(1)= getFriction(initPar.qbitHat0, initPar.thetaHat0, phi,n);

for time = 2:simLength+1
    timeSpan = time:stepSize:time+1;
   
    [yOut xtraOut] = ode4m(@(t, y) totalSystem(t, y, plantPar, tuningPar, qpump(mod(time,240)+1), qback, Pp(time-1), phi,n), timeSpan, y0);
    
    Pp(time)        = max(0,yOut(end, 1));
    Pc(time)        = max(plantPar.p0,yOut(end, 2));
    qbit(time)      = max(0,yOut(end, 3));
 %  xiHat(time)     = yOut(end, 4);
    thetaHat(:,time)= yOut(end,5:end);
    
    qchoke(time)    = xtraOut(end).var1;
    Pbit(time)      = xtraOut(end).var2;
    PbitHat(time)   = xtraOut(end).var3;
    qbitHat(time)   = xtraOut(end).var4;
    Zc(time)        = xtraOut(end).var5;
    PfricA(time)    = xtraOut(end).var6;
    PfricAHat(time) = xtraOut(end).var7;
    qbitAproc(time) = xtraOut(end).var8;
    PpDotRett(time) = xtraOut(end).var9;
    PpDotAproc(time)= xtraOut(end).var10;
    
    y0 = [yOut(end, 1) yOut(end, 2) yOut(end, 3) yOut(end,4) yOut(end,5:end)];

end   
end

function [dydt xtra] = totalSystem(t,y,plantPar, tuningPar, qpump, qback, PpLast, phi,n)

%Plant parameters
Vd      = plantPar.Vd;
betad   = plantPar.betad;
Va      = plantPar.Va;
betaa   = plantPar.betaa;
Ma      = plantPar.Ma;
Md      = plantPar.Md;
M       = Ma+Md;
Fd      = plantPar.Fd;

rhod    = plantPar.rhod;
rhoa    = plantPar.rhoa;
Kc      = plantPar.Kc;
P0      = plantPar.p0;
g       = plantPar.g;
hbit    = plantPar.hbit;

%qpump   = tuningPar.qpump;
%qback   = tuningPar.qback;
qres    = 0;

l1      = tuningPar.l1;

%Some defined constants
VaDot = 0;
a1 = betad/Vd;
a2 = 1/M;

%States
Pp      = max(0,y(1));
Pc      = max(P0,y(2));
qbit    = max(0,y(3));
xiHat   = y(4);

%theta3Hat=y(6);

thetaHat=y(5:end);
theta = tuningPar.theta;

PfricA = getFriction(qbit, theta, phi,n);

%pause
% Choke
Zc      = tuningPar.Zc;
Zc      = max(0,Zc);
Zc      = min(Zc, 1);
qchoke  = Kc*Zc*sqrt(2/rhoa*(Pc-P0));

% Plant

PpDot   = a1*(qpump - qbit);
PcDot   = betaa/Va*(qbit + qback - qchoke + qres - VaDot);
qbitDot = a2*(Pp - Pc -Fd*abs(qbit)*qbit -...
            PfricA + (rhod - rhoa)*g*hbit);

Pbit    = Pc + Ma*qbitDot + PfricA + rhoa*g*hbit;

%Observer
qbitHat = xiHat - l1*Pp;

PfricAHat = getFriction(qbitHat, thetaHat, phi,n);

xiHatDot= -l1*a1*(qbitHat-qpump) - a2*Fd*qbitHat^2 ...
        - a2*PfricAHat + a2*(rhod-rhoa)*g*hbit + a2*(Pp-Pc);

PbitHat = Pc + Ma*qbitDot + PfricAHat + rhoa*g*hbit;

        
phi1    = phi.phi1;
phi2    = phi.phi2;
phi3    = phi.phi3;
phi4    = phi.phi4;
if n == 10
    phi5    = phi.phi5;
    phi6    = phi.phi6;
    phi7    = phi.phi7;
    phi8    = phi.phi8;
    phi9    = phi.phi9;
    phi10   = phi.phi10;
end
%Passive identifier (15)
qbIndex = max(1,round(qbitHat*6e4));
qbIndex = min(2001, qbIndex);

%8 9 10 40
thetaHatDot(1) = -20*phi1(qbIndex)*(qbit-qbitHat);
thetaHatDot(2) = -30*phi2(qbIndex)*(qbit-qbitHat);
thetaHatDot(3) = -55*phi3(qbIndex)*(qbit-qbitHat);
thetaHatDot(4) = -60*phi4(qbIndex)*(qbit-qbitHat);
if n == 10
    thetaHatDot(5) = -60*phi5(qbIndex)*(qbit-qbitHat);
    thetaHatDot(6) = -60*phi6(qbIndex)*(qbit-qbitHat);
    thetaHatDot(7) = -60*phi7(qbIndex)*(qbit-qbitHat);
    thetaHatDot(8) = -55*phi8(qbIndex)*(qbit-qbitHat);
    thetaHatDot(9) = -50*phi9(qbIndex)*(qbit-qbitHat);
    thetaHatDot(10) = -100*phi10(qbIndex)*(qbit-qbitHat);
end
% Additional output variables
xtra.var1 = qchoke;
xtra.var2 = Pbit;
xtra.var3 = PbitHat;
xtra.var4 = qbitHat;
xtra.var5 = Zc;
xtra.var6 = PfricA;
xtra.var7 = PfricAHat;
xtra.var8 = qpump - 1/(a1)*(Pp-PpLast);
xtra.var9 = PpDot;
xtra.var10= Pp-PpLast;

%Collect the dynamtics
dydt = [PpDot; PcDot; qbitDot; xiHatDot; thetaHatDot'];

end