function [Pp, Pc, qbit, qchoke, Pbit, qbitHat, PbitHat, thetaHat, Zc, PfricA, PfricAHat, xiO] =...
    simulateStamnes(plantPar, tuningPar, initPar, qpump, qback, hbit, VaDot)


y0 = [initPar.Pp0 initPar.Pc0 initPar.qbit0 initPar.xiHat0 ...
    initPar.sigmaHat0]';

simLength = tuningPar.simLength;

Pp      = zeros(1, simLength+1);
Pc      = zeros(1, simLength+1);
Pbit    = zeros(1, simLength+1);
PbitHat = zeros(1, simLength+1);
qbit    = zeros(1, simLength+1);
qbitHat = zeros(1, simLength+1);
qchoke  = zeros(1, simLength+1);
thetaHat= zeros(4, simLength+1);
Zc      = zeros(1, simLength+1);
PfricA  = zeros(1, simLength+1);
PfricAHat= zeros(1, simLength+1);

Pp(1)       = initPar.Pp0;
Pc(1)       = initPar.Pc0;
qbit(1)     = initPar.qbit0;
qbitHat(1)  = initPar.qbitHat0;
thetaHat(:,1) = initPar.thetaHat0;
Pbit(1)     = 240;
PbitHat(1)  = 240;


simLength   = tuningPar.simLength;
stepSize    = tuningPar.stepSize;


g = 1;
qpSpan = 1:2000;
c = [0 500 1000 1500 2000 3000]; 
for j = qpSpan
        phi.phi1(g) = calcPhi(j,1,1,c);
        phi.phi2(g) = calcPhi(j,1,2,c);
        phi.phi3(g) = calcPhi(j,1,3,c);
        phi.phi4(g) = calcPhi(j,1,4,c);
        g = g+1;
end

PfricA(1)   = getFriction(initPar.qbit0, tuningPar.theta, phi,4);
PfricAHat(1)= getFriction(initPar.qbitHat0, initPar.thetaHat0, phi,4);


for time = 2:simLength+1
  %  clc
   % time-1
    timeSpan = time:stepSize:time+1;
    
    [yOut xtraOut] = ode4m(@(t, y) totalSystem(t, y, plantPar, tuningPar, qpump(mod(time,1600)+1), qback(mod(time,1600)+1), hbit,VaDot, phi), timeSpan, y0);
    
    Pp(time)        = max(1,yOut(end, 1));
    Pc(time)        = max(0,yOut(end, 2));
    qbit(time)      = max(0,yOut(end, 3));
  % xiHat(time)     = yOut(end, 4);
  % sigmaHat(time)  = yOut(end, 5-6);
    
    qchoke(time)    = xtraOut(end).var1;
    Pbit(time)      = xtraOut(end).var2;
    PbitHat(time)   = xtraOut(end).var3;
    qbitHat(time)   = xtraOut(end).var4;
    thetaHat(:,time)= xtraOut(end).var5;
    Zc(time)        = xtraOut(end).var6;
    PfricA(time)    = xtraOut(end).var7;
    PfricAHat(time) = xtraOut(end).var8;
    xiO(time)       = xtraOut(end).var9;
    
    y0 = [max(1,yOut(end, 1))
          max(0,yOut(end, 2))
          max(0,yOut(end, 3))
          yOut(end,4) 
          yOut(end,5) 
          yOut(end,6)
          yOut(end,7)
          yOut(end,8)]';
   
end   
end

function [dydt xtra] = totalSystem(t,y,plantPar, tuningPar, qpump, qback, hbit, VaDot,phi)
persistent eSum;
if(isempty(eSum))
    eSum = 0;
end

%Plant parameters
Vd      = plantPar.Vd;
betad   = plantPar.betad;
Va      = plantPar.Va;
betaa   = plantPar.betaa;
Ma      = plantPar.Ma;
Md      = plantPar.Md;
M       = Ma+Md;
Fd      = plantPar.Fd;
Fa      = plantPar.Fa;
rhod    = plantPar.rhod;
rhoa    = plantPar.rhoa;
Kc      = plantPar.Kc;
P0      = plantPar.p0;
g       = plantPar.g;
theta   = tuningPar.theta;
c = [0 500 1000 1500 2000 3000]; 
%Tuning parameters
%Zc      = tuningPar.Zc;

qres    = tuningPar.qres;

l1      = tuningPar.l1;
Gamma   = tuningPar.Gamma;


%Some defined constants
a1 = betad/Vd;
a2 = 1/M;

%States
Pp      = max(1,y(1));
Pc      = max(0,y(2));
qbit    = max(0,y(3));
xiHat   = y(4);
sigmaHat= [y(5);y(6); y(7); y(8)];


% Choke
PcRef   = tuningPar.PcRef;
e       = Pc - PcRef;
eSum    = eSum + e;
Kp      = tuningPar.Kp;
Ki      = tuningPar.Ki;
Zc      = Kp*e + Ki*eSum;
Zc      = max(0,Zc);
Zc      = min(Zc, 1);

qchoke  = Kc*Zc*sqrt(2/rhoa*(Pc-P0));

qbitLtr = qbit*6e4;
PfricA  = max(0,2.1e-9*qbitLtr^3 - 4.7e-6*qbitLtr^2 + 8.9e-3*qbitLtr-0.21);

% Plant
PpDot   = a1*(qpump - qbit);
PcDot   = betaa/Va*(qbit + qback -qchoke + qres - VaDot);
qbitDot = a2*(Pp - Pc -Fd*abs(qbit)*qbit -...
            PfricA + (rhod - rhoa)*g*hbit);

Pbit    = Pc + Ma*qbitDot + PfricA + rhoa*g*hbit;



%Observer
qbitHat = xiHat - l1*Pp;


eta         = Gamma*[calcPhiInt(qbitHat,1,c); 
                     calcPhiInt(qbitHat,2,c);
                     calcPhiInt(qbitHat,3,c);
                     calcPhiInt(qbitHat,4,c)];
                 
thetaHat    = sigmaHat - eta;

if (qbitHat <= 0 && ((Pp-Pc) < 0))
    xiHatDot = l1*a1*qpump;
else
xiHatDot= -l1*a1*qbitHat-Fd*a2*abs(qbitHat)*qbitHat-a2*getFriction(qbitHat,thetaHat,phi,4)*sign(qbitHat)...
    + a2*(Pp-Pc)+l1*a1*qpump;
end



PbitHat = Pc + Ma*(a2*(Pp - Pc -Fd*abs(qbitHat)*qbitHat -...
    getFriction(qbitHat,thetaHat,phi,4)*sign(qbitHat)) + 0*hbit) + getFriction(qbitHat,thetaHat,phi,4)*sign(qbitHat) + rhod*g*hbit;

       
%Adaptation
qbitIdx = round(max(1,min(qbitHat*6e4,2000)));

sigmaHatDot = -l1*Gamma*[phi.phi1(qbitIdx); 
                          phi.phi2(qbitIdx); 
                          phi.phi3(qbitIdx); 
                          phi.phi4(qbitIdx)]...
                 *(-a1*qbitHat+a1*qpump) ...
                 + Gamma*[phi.phi1(qbitIdx);  
                          phi.phi2(qbitIdx); 
                          phi.phi3(qbitIdx); 
                          phi.phi4(qbitIdx)]*xiHatDot;

if qbitHat*6e4 < 0
  %  sigmaHatDot = [0 0 0 0]';
end
% Additional output variables
xtra.var1 = qchoke;
xtra.var2 = Pbit;
xtra.var3 = PbitHat;
xtra.var4 = qbitHat;
xtra.var5 = thetaHat;
xtra.var6 = Zc;
xtra.var7 = PfricA;
xtra.var8 = getFriction(qbitHat, thetaHat, phi,4)*sign(qbitHat);
xtra.var9 = xiHat;

%Collect the dynamtics
if (Pp <=1 && PpDot < 0)
    PpDot = 0;
end
if (Pc <= 0 && PcDot < 0)
    PcDot = 0;
end
if( qbit <= 0 && qbitDot < 0)
    qbitDot = 0;
end

dydt = [PpDot; PcDot; qbitDot; xiHatDot; sigmaHatDot];

end