% Find the pressure curve
clear all;
load 'fricData.mat';
i = [1615; %indexes in data
     1680;
     1770;
     1848;
     1929;
     2008;
     2154;
     2220;
     2306;
     2404;
     2498;
     2575;
     2651;
     2721;
     2802;
     2868;
     2957;
     3005;
     3157;
     3228;
     3314;
     3408;
     3524;
     3620;
     3712;
     3851;
     3977;
     4097;
     4219;
     4325;
     4426;
     4531;
     4652;
     4777;
     4879;
     4968;
     5059;
     5160];
j = ceil(i/5)+730; %indexes in pbit

qp      = data(i,2);
qback   = data(i,5);
qc      = data(i, 38);
pp      = data(i, 16);
pc      = data(i, 36);
pbit    = pbitMeas(j,1);
phydro  = 263.6061;


%% friction curves
figure
plot(qp,pp-pbit+phydro,'o');
%legend('Drillstring');
ylabel('p_{fric}^d [bar]');
xlabel('q_p [ltr/min]');
grid on
axis([0 2100 0 190]);
set(gcf, 'PaperSize', [15 10]);
set(gcf, 'PaperPosition', [0 0 15 10]);
%pause
saveas(gcf,'C:\Master\LaTeX\fig\PFricD','pdf')

figure
plot(qp,pbit-pc-phydro,'r x');
%legend('Annulus');
ylabel('p_{fric}^a [bar]');
xlabel('q_p [ltr/min]');
grid on
axis([0 2100 0 25]);
set(gcf, 'PaperSize', [15 10]);
set(gcf, 'PaperPosition', [0 0 15 10]);
%pause
saveas(gcf,'C:\Master\LaTeX\fig\PFricA','pdf')
close all
%% flows
figure
plot(i,qp,'o', i, qc,'r x',i,qback,'k *');
legend('q_p', 'q_c', 'q_{back}');
ylabel('Flow [ltr/min]');
xlabel('Meas no');
title('Flows');
grid on
%% curvefit drill string
figure
plot(qp,pp-pbit+phydro,'x', qp, 1.4e-4*qp.^1.86,'r', qp, 3.3e-3*qp+2.1+4.3e-5*qp.^2);
legend('p_{fric}^d','1.4*10^{-4}q_p^{1.86}','4.3*10^{-5}q_p^2+3.3*10^{-3}q_p+2.1');
ylabel('Friction [bar]');
xlabel('q_p [ltr/min]');
title('Curve fit for drillstring.');
grid on

%% curvefit annulus
figure
plot(qp,pbit-pc-phydro,'x', qp, 1.25e-5*qp.^1.86,'r', qp, 1.7e-6*qp.^2+4e-3*qp+0.5);
legend('p_{fric}^a','1.25*10^{-5}q_p^{1.86}','1.7*10^{-6}q_p^2+4*10^{-3}q_p+0.5');
ylabel('Friction [bar]');
xlabel('q_p [ltr/min]');
title('Curve fit for annulus.');
grid on

%% flows in vs out
figure
plot(qp,qc)
title('pump- vs choke flow.');
xlabel('q_p [ltr/min]');
ylabel('q_c [ltr/min]');
grid on;

%% friction curves, separate plots
figure
plot(qp,pp-pbit+phydro,'o');
ylabel('p_{fric}^d [bar]');
xlabel('q_p [ltr/min]');
grid on

%%
figure
plot(qp,pbit-pc-phydro,'r x', qp, 2.1e-9*qp.^3 - 4.7e-6*qp.^2+0.0089*qp-.21);
ylabel('Pressure [bar]');
xlabel('q_{bit} [ltr/min]');
legend('p_{fric}^a','2.1*10^{-9}q_{bit}^3-4.7*10^{-6}q_{bit}^2+8.9*10^{-3}q_{bit}-0.21');
grid on


%% qp
figure
hold on
plot(1615:5160,data(1615:5160,10));
plot(i, qp,'r x', 'MarkerSize',10,'LineWidth',1.5);
grid on;
legend('q_p','Measurements used');
xlabel('Measurement number');
ylabel('Flow [ltr/min]');
hold off


%% friction curves, with quadratic friction
figure
plot(qp,pp-pbit+phydro,'o', qp, 0.16500e-2*(qp./6).^2);
ylabel('p_{fric}^d [bar]');
xlabel('q_p [ltr/min]');
legend('p_{fric}^d', 'F_dq_p^2');
grid on
axis([0 2100 0 190]);
set(gcf, 'PaperSize', [15 10]);
set(gcf, 'PaperPosition', [0 0 15 10]);
%pause
%saveas(gcf,'C:\Master\LaTeX\fig\PFricDcurvefit','pdf')
%%
figure
plot(qp,pbit-pc-phydro,'r x', qp, 0.02080e-2*(qp./6).^2);
ylabel('p_{fric}^a [bar]');
xlabel('q_p [ltr/min]');
legend('p_{fric}^a', 'F_aq_p^2');
grid on
axis([0 2100 0 25]);
set(gcf, 'PaperSize', [15 10]);
set(gcf, 'PaperPosition', [0 0 15 10]);
%pause
%saveas(gcf,'C:\Master\LaTeX\fig\PFricAcurvefit','pdf')
%% yvindplott
%alpha = p_fric^a/(p_fric^d+p_fric^a) langs vertikal akse og flow langs
%horisontal aksen
PfricA = pbit-pc-phydro;
PfricD = pp-pbit+phydro;
alpha = PfricA./(PfricD+PfricA);

figure
plot(qp, alpha);
xlabel('Flow [ltr/min]');
ylabel('Alpha');
grid on

%%











