%% Function to run poly. estimation and kalman-filter
clear all;
close all;
clc;

%Get constants
plantPar    = getPlantPar();
tuningPar   = getTuningPar();
initPar     = getInitPar(plantPar, tuningPar);
[qpump qback hbit VaDot] = makeCase();

%Run simulation
%[PpK, PcK, qbitK, qchokeK, PbitK, xK, xHatK, Zc1]...
 %   = simulateKalman(plantPar, tuningPar, initPar, qpump, qback);
[PpP, PcP, qbitP, qchokeP, PbitP, xP, xHatP, Zc2]...
    = simulatePoly(plantPar, tuningPar, initPar, qpump, qback, hbit, VaDot);

qpumpL = [qpump qpump];
qbackL = [qback qback];
hbitL  = [hbit hbit];
VaDotL  = [VaDot VaDot];


%PLOTS
time = 0:tuningPar.simLength;
M = plantPar.Ma + plantPar.Md;
g = plantPar.g;
rhod = plantPar.rhod;
rhoa = plantPar.rhoa;
Fd = plantPar.Fd;
Fa = plantPar.Fa;
F = Fd + Fa;
H = tuningPar.H;
%% Plots for latex
figure();
plot(time, qpumpL(1:time(end)+1)*6e4, time, qbackL(1:time(end)+1)*6e4,time, Zc2*1000);
xlabel('Time [s]');
ylabel('[-]');
legend('q_{p} [ltr/min]', 'q_{b} [ltr/min]', '1000*Zc');
grid on;
axis([0 1600 -56 1550]);
set(gcf,'PaperSize',[15,4]);
%%
figure();
plot(time, Pbit, time, PbitHat,'r--');
xlabel('Time [s]');
ylabel('Pressure [Bar]');
legend('p_{bit}', 'p_{bit}Hat');
grid on;
axis([0 1600 220 260]);
set(gcf,'PaperSize',[15,4]);
%%
figure
plot(time, qbitP*6e4, time, H(2,2)*xHatP(2,:)*6e4,'r--');
xlabel('Time [s]');
ylabel('Flow [l/min]');
legend('q_{bit}', 'q_{bit}Hat');
grid on;
axis([0 1600 -50 1550])
set(gcf,'PaperSize',[15,4]);
%%
figure();
plot(time, (Fa+Fd)/M*ones(1,tuningPar.simLength+1), time, H(3,3)*xHatP(3,:)/M,'r--');
xlabel('Time [s]');
ylabel('');
legend('\theta_1','\theta_1 Hat');
grid on;
axis([0 1600 22 28]);
set(gcf,'PaperSize',[15,4]);
%%
% figure();
% plot(time, (rhod-rhoa)*g/M*ones(1,tuningPar.simLength+1), time, thetaHat(2,:),'r--');
% xlabel('Time [s]');
% ylabel('');
% legend('\theta_2','\theta_2 Hat');
% grid on;
% axis([0 1600 -8.5e-6 1e-6])
% set(gcf,'PaperSize',[15,4]);

