%% Function to run simulation and make plots
clear all;
close all;
clc;

%Get constants
plantPar    = getPlantPar();
tuningPar   = getTuningPar();
initPar     = getInitPar(plantPar, tuningPar);
[qpump qback hbit VaDot] = makeCase();
tf1 = 1;
tf2 = 10;
tf3 = 20;
%Run simulation
[Pp, Pc, qbit, qchoke, Pbit, qbitHat, PbitHat, thetaHat, Zc]...
    = simulateKaasa(plantPar, tuningPar, initPar, qpump, qback, hbit, VaDot, tf1);
disp('done');


qpumpL = [qpump qpump];
qbackL = [qback qback];
hbitL  = [hbit hbit];
VaDotL  = [VaDot VaDot];


%PLOTS
time = 0:tuningPar.simLength;
M = plantPar.Ma + plantPar.Md;
g = plantPar.g;
rhod = plantPar.rhod;
rhoa = plantPar.rhoa;
Fd = plantPar.Fd;
Fa = plantPar.Fa;
F = Fd + Fa;
%%
figure
plot(1:length(qpump), qpump*6e4,1:length(qback), qback*6e4);
axis([0 length(qpump), 0 2100]);
legend('q_{p}', 'q_{back}');
xlabel('Time [s]');
ylabel('Flow [ltr/min]');
grid on
set(gcf, 'PaperUnits', 'centimeters');
set(gcf, 'PaperSize', [10 10]);
set(gcf, 'PaperPosition', [0 0 10 10]);
%pause
%saveas(gcf,'C:\Master\LaTeX\fig\fricCaseAddSimqpump','pdf')

%% Pressures
figure();
subplot(211),plot(time, Pp,time, 10*Pc);
ylabel('Pressure [Bar]');
xlabel('Time [s]');
legend('Pp', '10*Pc');
grid on;


subplot(212),plot(time, Pbit, time, PbitHat);
xlabel('Time [s]');
ylabel('Pressure [Bar]');
legend('p_{bit}', 'p_{bit}Hat');
grid on;

%%
% Flows
figure();
subplot(311), plot(time, qpumpL(1:time(end)+1)*6e4, time, qbackL(1:time(end)+1)*6e4);
xlabel('Time [s]');
ylabel('Flow [l/min]');
legend('q_{pump}', 'q_{back}');
grid on;

subplot(312), plot(time, qbit*60e3, time, qbitHat*6e4,'r--');
xlabel('Time [s]');
ylabel('Flow [l/min]');
legend('q_{bit}', 'q_{bit}Hat');
grid on;

subplot(313), plot(time, qchoke*60e3, time, Zc*1000);
xlabel('Time [s]');
ylabel('Flow [l/min]');
legend('q_{choke}', 'Zc');
grid on;

%% Estimation

figure();
subplot(311), plot(time, qbit*6e4, time, qbitHat*6e4,'r--');
xlabel('Time [s]');
ylabel('Flow [m^3/h]');
legend('q_{bit}','q_{bit}Hat');
grid on;

subplot(312), plot(time, (Fa+Fd)/M*ones(1,tuningPar.simLength+1), time, thetaHat(1,:),'r--');
xlabel('Time [s]');
ylabel('[]');
legend('\theta_1','\theta_1 Hat');
grid on;
subplot(313), plot(time, (rhod-rhoa)*g/M*ones(1,tuningPar.simLength+1), time, thetaHat(2,:),'r--');
xlabel('Time [s]');
ylabel('[]');
legend('\theta_2','\theta_2 Hat');
%axis([0 time(end) -1e-6 1e-6]);
grid on;



%% Error

% figure();
% subplot(311), plot(time, qbit-qbitHat);
% xlabel('Time [s]');
% ylabel('Value [?]');
% legend('qbit-qbitHat');
% grid on;
% 
% 
% subplot(312), plot(time, (Fa+Fd)/M*ones(1,tuningPar.simLength+1)-(thetaHat(1,:)));
% xlabel('Time [s]');
% ylabel('Value [?]');
% legend('F-FHat');
% grid on;
% subplot(313), plot(time, 0*ones(1,tuningPar.simLength+1)-(thetaHat(2,:)));
% xlabel('Time [s]');
% ylabel('Value [?]');
% legend('rho-rhoHat');
% grid on;

%% Plots for latex
figure();
plot(time, qpumpL(1:time(end)+1)*6e4, time, qbackL(1:time(end)+1)*6e4,time, Zc*1000);
xlabel('Time [s]');
ylabel('[-]');
legend('q_{p} [ltr/min]', 'q_{b} [ltr/min]', '1000*Zc');
grid on;
axis([0 1600 -56 1550]);
set(gcf,'PaperSize',[15,4]);
%%
figure();
plot(time, Pbit, time, PbitHat,'--');
xlabel('Time [s]');
ylabel('Pressure [Bar]');
legend('p_{bit}', 'p_{bit}Hat');
grid on;
axis([0 1600 220 260]);
set(gcf,'PaperSize',[15,4]);
%%
figure
plot(time, qbit*6e4, time, qbitHat*6e4,'--');
xlabel('Time [s]');
ylabel('Flow [l/min]');
legend('q_{bit}', 'q_{bit}Hat');
grid on;
%axis([0 1600 -50 1550])
set(gcf,'PaperSize',[15,4]);
%%
figure();
plot(time, (Fa+Fd)/M*ones(1,tuningPar.simLength+1), time, thetaHat(1,:),'--');
xlabel('Time [s]');
ylabel('[-]');
legend('\theta_1','\theta_1 Hat');
grid on;
axis([0 1600 18 28]);
set(gcf,'PaperSize',[15,4]);
%%
figure();
plot(time, (rhod-rhoa)*g/M*ones(1,tuningPar.simLength+1), time, thetaHat(2,:),'--');
xlabel('Time [s]');
ylabel('[-]');
legend('\theta_2','\theta_2 Hat');
grid on;
axis([0 1600 -8.5e-6 1e-6])
set(gcf,'PaperSize',[15,4]);

