%% Function to run simulation and make plots
clear all;
close all;
clc;

%Get constants
plantPar    = getPlantPar();
tuningPar   = getTuningPar();
initPar     = getInitPar(plantPar, tuningPar);
[qpump qback hbit VaDot] = makeCase();
tf1 = 1;
tf2 = 10;
tf3 = 20;
%Run simulation
[Pp, Pc, qbit, qchoke, Pbit, qbitHat, PbitHat, thetaHat, Zc]...
    = simulateKaasa(plantPar, tuningPar, initPar, qpump, qback, hbit, VaDot, tf1);
disp('done');
[Pp1, Pc1, qbit1, qchoke1, Pbit1, qbitHat1, PbitHat1, thetaHat1, Zc1]...
    = simulateKaasa(plantPar, tuningPar, initPar, qpump, qback, hbit, VaDot,tf2);
disp('done');
[Pp2, Pc2, qbit2, qchoke2, Pbit2, qbitHat2, PbitHat2, thetaHat2, Zc2]...
    = simulateKaasa(plantPar, tuningPar, initPar, qpump, qback, hbit, VaDot,tf3);
disp('done');

qpumpL = [qpump qpump];
qbackL = [qback qback];
hbitL  = [hbit hbit];
VaDotL  = [VaDot VaDot];


%PLOTS
time = 0:tuningPar.simLength;
M = plantPar.Ma + plantPar.Md;
g = plantPar.g;
rhod = plantPar.rhod;
rhoa = plantPar.rhoa;
Fd = plantPar.Fd;
Fa = plantPar.Fa;
F = Fd + Fa;

%% Pressures
figure();
subplot(211),plot(time, Pp,time, 10*Pc);
ylabel('Pressure [Bar]');
xlabel('Time [s]');
legend('Pp', '10*Pc');
grid on;


subplot(212),plot(time, Pbit, time, PbitHat);
xlabel('Time [s]');
ylabel('Pressure [Bar]');
legend('p_{bit}', 'p_{bit}Hat');
grid on;

%%
% Flows
figure();
subplot(311), plot(time, qpumpL(1:time(end)+1)*6e4, time, qbackL(1:time(end)+1)*6e4);
xlabel('Time [s]');
ylabel('Flow [l/min]');
legend('q_{pump}', 'q_{back}');
grid on;

subplot(312), plot(time, qbit*60e3, time, qbitHat*6e4,'r--');
xlabel('Time [s]');
ylabel('Flow [l/min]');
legend('q_{bit}', 'q_{bit}Hat');
grid on;

subplot(313), plot(time, qchoke*60e3, time, Zc*1000);
xlabel('Time [s]');
ylabel('Flow [l/min]');
legend('q_{choke}', 'Zc');
grid on;

%% Estimation

figure();
subplot(311), plot(time, qbit*6e4, time, qbitHat*6e4,'r--');
xlabel('Time [s]');
ylabel('Flow [m^3/h]');
legend('q_{bit}','q_{bit}Hat');
grid on;

subplot(312), plot(time, (Fa+Fd)/M*ones(1,tuningPar.simLength+1), time, thetaHat(1,:),'r--');
xlabel('Time [s]');
ylabel('[]');
legend('\theta_1','\theta_1 Hat');
grid on;
subplot(313), plot(time, (rhod-rhoa)*g/M*ones(1,tuningPar.simLength+1), time, thetaHat(2,:),'r--');
xlabel('Time [s]');
ylabel('[]');
legend('\theta_2','\theta_2 Hat');
%axis([0 time(end) -1e-6 1e-6]);
grid on;



%% Error

% figure();
% subplot(311), plot(time, qbit-qbitHat);
% xlabel('Time [s]');
% ylabel('Value [?]');
% legend('qbit-qbitHat');
% grid on;
% 
% 
% subplot(312), plot(time, (Fa+Fd)/M*ones(1,tuningPar.simLength+1)-(thetaHat(1,:)));
% xlabel('Time [s]');
% ylabel('Value [?]');
% legend('F-FHat');
% grid on;
% subplot(313), plot(time, 0*ones(1,tuningPar.simLength+1)-(thetaHat(2,:)));
% xlabel('Time [s]');
% ylabel('Value [?]');
% legend('rho-rhoHat');
% grid on;

%% Plots for latex
figure();
plot(time, qpumpL(1:time(end)+1)*6e4, time, qbackL(1:time(end)+1)*6e4,time, Zc2*1000);
xlabel('Time [s]');
ylabel('[-]');
legend('q_{p} [ltr/min]', 'q_{b} [ltr/min]', '1000*Zc');
grid on;
axis([0 1600 -56 1550]);
set(gcf,'PaperSize',[15,4]);
%%
figure();
plot(time, Pbit, time, PbitHat,'--', time, PbitHat1,'--', time, PbitHat2,'--');
xlabel('Time [s]');
ylabel('Pressure [Bar]');
legend('p_{bit}', ['p_{bit}Hat \tau_f=' num2str(tf1)], ['p_{bit}Hat \tau_f=' num2str(tf2)], ['p_{bit}Hat \tau_f=' num2str(tf3)]);
grid on;
axis([0 1600 220 260]);
set(gcf,'PaperSize',[15,4]);
%%
figure
plot(time, qbit*6e4, time, qbitHat*6e4,'--', time, qbitHat1*6e4,'--', time, qbitHat2*6e4,'--');
xlabel('Time [s]');
ylabel('Flow [l/min]');
legend('q_{bit}', ['q_{bit}Hat \tau_f=' num2str(num2str(tf1))], ['q_{bit}Hat \tau_f=' num2str(tf2)], ['q_{bit}Hat \tau_f=' num2str(tf3)]);
grid on;
%axis([0 1600 -50 1550])
set(gcf,'PaperSize',[15,4]);
%%
figure();
plot(time, (Fa+Fd)/M*ones(1,tuningPar.simLength+1), time, thetaHat(1,:),'--',...
    time, thetaHat1(1,:),'--', time, thetaHat2(1,:),'--');
xlabel('Time [s]');
ylabel('[-]');
legend('\theta_1',['\theta_1 Hat \tau_f=' num2str(tf1)], ['\theta_1 Hat\tau_f=' num2str(tf2)], ['\theta_1 Hat \tau_f=' num2str(tf3)]);
grid on;
axis([0 1600 18 28]);
set(gcf,'PaperSize',[15,4]);
%%
figure();
plot(time, (rhod-rhoa)*g/M*ones(1,tuningPar.simLength+1), time, thetaHat(2,:),'--',...
     time,  thetaHat1(2,:),'--', time, thetaHat2(2,:),'--');
xlabel('Time [s]');
ylabel('[-]');
legend('\theta_2',['\theta_2 Hat \tau_f=' num2str(tf1)], ['\theta_2 Hat\tau_f=' num2str(tf2)], ['\theta_2 Hat \tau_f=' num2str(tf3)]);
grid on;
axis([0 1600 -8.5e-6 1e-6])
set(gcf,'PaperSize',[15,4]);

