function [Pp, Pc, qbit, qchoke, Pbit, qbitHat, PbitHat, thetaHat, Zc] =...
    simulateGrip(plantPar, tuningPar, initPar, qpump, qback, hbit, VaDot)


y0 = [initPar.Pp0 initPar.Pc0 initPar.qbit0 initPar.thetaHat0(1) initPar.thetaHat0(2) initPar.qbitHat0]';

simLength = tuningPar.simLength;

Pp      = zeros(1, simLength+1);
Pc      = zeros(1, simLength+1);
Pbit    = zeros(1, simLength+1);
PbitHat = zeros(1, simLength+1);
qbit    = zeros(1, simLength+1);
qchoke  = zeros(1, simLength+1);
thetaHat= zeros(2, simLength+1);
qbitHat = zeros(1, simLength+1);
Zc      = zeros(1, simLength+1);


Pp(1)       = initPar.Pp0;
Pc(1)       = initPar.Pc0;
qbit(1)     = initPar.qbit0;
qbitHat(1)  = initPar.qbitHat0;
thetaHat(:,1) = initPar.thetaHat0;
Pbit(1)     = 240;
PbitHat(1)  = 240;


simLength   = tuningPar.simLength;
stepSize    = tuningPar.stepSize;


for time = 2:simLength+1
    timeSpan = time:stepSize:time+1;

    [yOut xtraOut] = ode4m(@(t, y) totalSystem(t, y, plantPar, tuningPar, qpump(mod(time,1601)+1), qback(mod(time,1601)+1), Pp(time-1), hbit(mod(time,1601)+1), VaDot(mod(time,1601)+1)), timeSpan, y0);
    
    Pp(time)        = max(1,yOut(end,1));
    Pc(time)        = max(0,yOut(end,2));
    qbit(time)      = max(0,yOut(end,3));
    
    thetaHat(:,time)= yOut(end,4:5);
    qbitHat(time) = yOut(end,6);
       
    qchoke(time)    = xtraOut(end).var1;
    Pbit(time)      = xtraOut(end).var2;
    PbitHat(time)   = xtraOut(end).var3;
    Zc(time)        = xtraOut(end).var4;

    y0 = [max(1,yOut(end, 1))
          max(0,yOut(end, 2))
          max(0,yOut(end, 3))
          yOut(end,4) 
          yOut(end,5) 
          yOut(end,6)]';

end   
end

function [dydt xtra] = totalSystem(t,y,plantPar, tuningPar, qpump, qback, PpLast, hbit, VaDot)
persistent eSum;
if(isempty(eSum))
    eSum = 0;
end

%Plant parameters
Vd      = plantPar.Vd;
betad   = plantPar.betad;
Va      = plantPar.Va;
betaa   = plantPar.betaa;
Ma      = plantPar.Ma;
Md      = plantPar.Md;
M       = Ma+Md;
Fd      = plantPar.Fd;
Fa      = plantPar.Fa;
F       = Fa+Fd;
rhod    = plantPar.rhod;
rhoa    = plantPar.rhoa;
Kc      = plantPar.Kc;
P0      = plantPar.p0;
g       = plantPar.g;


%Tuning parameters
qres    = tuningPar.qres;

gamma0  = tuningPar.gamma0;
Gamma   = tuningPar.GammaGBig;


%Some defined constants
a1 = betad/Vd;
a2 = 1/M;

%States
Pp      = max(1,y(1));
Pc      = max(0,y(2));
qbit    = max(0,y(3));
thetaHat= [y(4);y(5)];
qbitHat = y(6);


% Choke
PcRef   = tuningPar.PcRef;
e       = Pc - PcRef;
eSum    = eSum + e;
Kp      = tuningPar.Kp;
Ki      = tuningPar.Ki;
Zc      = Kp*e + Ki*eSum;
Zc      = max(0,Zc);
Zc      = min(Zc, 1);

qchoke  = Kc*Zc*sqrt(2/rhoa*(Pc-P0));

% Plant
PpDot   = a1*(qpump - qbit);

PcDot   = betaa/Va*(qbit + qback -qchoke + qres - VaDot);
qbitDot = a2*(Pp - Pc -(Fa+Fd)*abs(qbit)*qbit + (rhod-rhoa)*g*hbit);

Pbit    = Pc + Ma*qbitDot + Fa*abs(qbit+qres)*(qbit+qres) + rhoa*g*hbit;

%System
  %Make qbit from known signals
  PpDotEst  = Pp-PpLast;
  qbitKnown = qpump - 1/a1*PpDotEst;

f0      = a2*(Pp - Pc);
pfi     = [-abs(qbitKnown)*qbitKnown;
            hbit];

%Grip observer
thetaHatDot = Gamma*pfi*gamma0*(qbitKnown - qbitHat);
qbitHatDot  = f0 + gamma0*(qbitKnown-qbitHat) + pfi'*thetaHat + inv(gamma0)*pfi'*thetaHatDot;


PbitHat = Pc + Ma*(a2*(Pp-Pc)-thetaHat(1)*abs(qbitHat)*qbitHat+thetaHat(2)*hbit)...
            + (M*thetaHat(1)-Fd)*abs(qbitHat)*qbitHat + (rhod*g - M*thetaHat(2))*hbit;


%Additional output variables
xtra.var1 = qchoke;
xtra.var2 = Pbit;
xtra.var3 = PbitHat;
xtra.var4 = Zc;
%xtra.var5 = thetaHat;
%xtra.var6 = 0;%PpDot;
xtra.var7 = pfi'*([F/M;0]-thetaHat);%PpDotEst;


%Collect the dynamtics
dydt = [PpDot; PcDot; qbitDot; thetaHatDot(1); thetaHatDot(2); qbitHatDot];

end