function [Pp, Pc, qbit, qchoke, Pbit, qbitHat, PbitHat, thetaHat, Zc] =...
    simulateKaasa(plantPar, tuningPar, initPar, qpump, qback, hbit, VaDot, tf)


y0 = [initPar.Pp0 initPar.Pc0 initPar.qbit0 initPar.xiHat0 ...
     initPar.xf0 initPar.thetaHat0(1) initPar.thetaHat0(2)]';

simLength = tuningPar.simLength;

Pp      = zeros(1, simLength+1);
Pc      = zeros(1, simLength+1);
Pbit    = zeros(1, simLength+1);
PbitHat = zeros(1, simLength+1);
qbit    = zeros(1, simLength+1);
qbitHat = zeros(1, simLength+1);
qchoke  = zeros(1, simLength+1);
thetaHat= zeros(2, simLength+1);
Zc      = zeros(1, simLength+1);

Pp(1)       = initPar.Pp0;
Pc(1)       = initPar.Pc0;
qbit(1)     = initPar.qbit0;
qbitHat(1)  = initPar.qbitHat0;
thetaHat(:,1) = initPar.thetaHat0;
Pbit(1)     = 240;
PbitHat(1)  = 240;

simLength   = tuningPar.simLength;
stepSize    = tuningPar.stepSize;


for time = 2:simLength+1
    timeSpan = time:stepSize:time+1;
    
    [yOut xtraOut] = ode4m(@(t, y) totalSystem(t, y, plantPar, tuningPar, qpump(mod(time,1601)+1), qback(mod(time,1601)+1), Pp(time-1),hbit(mod(time,1601)+1),VaDot(mod(time,1601)+1),tf), timeSpan, y0);
    
    Pp(time)        = max(1,yOut(end, 1));
    Pc(time)        = max(0,yOut(end, 2));
    qbit(time)      = max(0,yOut(end, 3));
 %  xiHat(time)     = yOut(end, 4);
%   xf(time)        = yOut(end,5);
    thetaHat(:,time)= yOut(end,6:7);
    
    qchoke(time)    = xtraOut(end).var1;
    Pbit(time)      = xtraOut(end).var2;
    PbitHat(time)   = xtraOut(end).var3;
    qbitHat(time)   = xtraOut(end).var4;
    Zc(time)        = xtraOut(end).var5;
  %  PpSjekk(time)   = xtraOut(end).var6;
  %  PpDot(time)     = xtraOut(end).var7;
    
    y0 = [max(1,yOut(end,1))
          max(0,yOut(end,2)) 
          max(0,yOut(end,3))
          yOut(end,4)
          yOut(end,5)
          yOut(end,6) 
          yOut(end,7)];

end   
end

function [dydt xtra] = totalSystem(t,y,plantPar, tuningPar, qpump, qback, PpLast,hbit, VaDot, tf)

persistent eSum;
if(isempty(eSum))
    eSum = 0;
end


%Plant parameters
Vd      = plantPar.Vd;
betad   = plantPar.betad;
Va      = plantPar.Va;
betaa   = plantPar.betaa;
Ma      = plantPar.Ma;
Md      = plantPar.Md;
M       = Ma+Md;
Fd      = plantPar.Fd;
Fa      = plantPar.Fa;
rhod    = plantPar.rhod;
rhoa    = plantPar.rhoa;
Kc      = plantPar.Kc;
P0      = plantPar.p0;
g       = plantPar.g;


qres    = tuningPar.qres;

l1      = tuningPar.l1;
Gamma   = tuningPar.Gamma;


%tf      = tuningPar.tf;


%Some defined constants
a1 = betad/Vd;
a2 = 1/M;

%States
Pp      = max(1,y(1));
Pc      = max(0,y(2));
qbit    = max(0,y(3));
xiHat   = y(4);
xf      = y(5);
thetaHat=[y(6); y(7)];

% Choke
PcRef   = tuningPar.PcRef;
e       = Pc - PcRef;
eSum    = eSum + e;
Kp      = tuningPar.Kp;
Ki      = tuningPar.Ki;
Zc      = Kp*e + Ki*eSum;
Zc      = max(0,Zc);
Zc      = min(Zc, 1);
qchoke  = Kc*Zc*sqrt(2/rhoa*(Pc-P0));

% Plant
PpDot   = a1*(qpump - qbit);
PcDot   = betaa/Va*(qbit + qback - qchoke + qres - VaDot);
qbitDot = a2*(Pp - Pc -Fd*abs(qbit)*qbit -...
            Fa*abs(qbit + qres)*(qbit+qres) + (rhod - rhoa)*g*hbit);

Pbit    = Pc + Ma*qbitDot + Fa*abs(qbit + qres)*(qbit+qres) + rhoa*g*hbit;

%Observer
qbitHat = xiHat - l1*Pp;

if qbitHat < 0 && Pp < Pc
    xiHatDot = l1*a1*qpump;
    PbitHat = Pc + (rhod*g - M*thetaHat(2))*hbit;
else
    
    xiHatDot= -l1*a1*qbitHat-thetaHat(1)*abs(qbitHat)*qbitHat...
            + thetaHat(2)*hbit+a2*(Pp-Pc)+l1*a1*qpump;
        
PbitHat = Pc + Ma*(a2*(Pp-Pc)-thetaHat(1)*abs(qbitHat)*qbitHat+thetaHat(2)*hbit)...
            + (M*thetaHat(1)-Fd)*abs(qbitHat)*qbitHat + (rhod*g - M*thetaHat(2))*hbit;
end
        
%Identifier driven by filtered error (lgn. 20-22)
%qfTld       = 1/tf*(xf - Pp/a1);
phi         = [-abs(qbitHat)*qbitHat; hbit];
%thetaHatDot = Gamma*phi*qfTld; %Filtered version (20)

xfDot       = 0;%-qfTld + qpump - qbitHat;

%Passive identifier (15)
thetaHatDot = Gamma*phi*(qpump - 1/(a1)*(Pp-PpLast)-qbitHat);


% Additional output variables
xtra.var1 = qchoke;
xtra.var2 = Pbit;
xtra.var3 = PbitHat;
xtra.var4 = qbitHat;
xtra.var5 = Zc;


%Collect the dynamtics
dydt = [PpDot; PcDot; qbitDot; xiHatDot; xfDot; thetaHatDot(1); thetaHatDot(2)];

end