%% Function to run simulation and make plots
clear all;
close all;
clc;

%Get constants
plantPar    = getPlantPar();
tuningPar   = getTuningPar();
initPar     = getInitPar(plantPar, tuningPar);
[qpump qback]= getqpump(tuningPar.simLength);
%Run simulation
[PpK, PcK, qbitK, qchokeK, PbitK, qbitHatK, PbitHatK, thetaHatK, PcHatK, theta3HatK,Zc, PpSjekk, PpDot]...
    = simulateKaasa(plantPar, tuningPar, initPar, qpump, qback);
[PpS, PcS, qbitS, qchokeS, PbitS, qbitHatS, PbitHatS, thetaHatS, PcHatS, theta3HatS]...
    = simulateStamnes(plantPar, tuningPar, initPar, qpump, qback);


%PLOTS
time = 0:tuningPar.simLength;

%% Pressures
figure();
subplot(211),plot(time, PpK,time, 10*PcK);
ylabel('Pressure [Bar]');
xlabel('Time [s]');
legend('Pp', '10*Pc');
grid on;


subplot(212),plot(time, PbitK, time, PbitHatS, 'r--', time, PbitHatK, 'g--');
xlabel('Time [s]');
ylabel('Pressure [Bar]');
legend('Pbit', 'PbitHatS', 'PbitHatK');
axis([0 tuningPar.simLength 200 250]);
grid on;

%% Flows
figure();
subplot(311), plot(time, qpump*6e4, time, qback*6e4);
xlabel('Time [s]');
ylabel('Flow [l/min]');
legend('qpump', 'qback');
grid on;

subplot(312), plot(time, qbitS*60e3, time, qbitHatS*60e3,'r--', time, qbitHatK*60e3,'g--');
xlabel('Time [s]');
ylabel('Flow [l/min]');
legend('qbit', 'qbitHatS', 'qbitHatK');
grid on;

subplot(313), plot(time, qchokeS*60e3, time, 1000*Zc);
xlabel('Time [s]');
ylabel('Flow [l/min]');
legend('qchoke', '1000*Zc');
grid on;

%% Adaptation

%The correct values...
theta1 = ones(1,tuningPar.simLength+1)*(plantPar.Fd+plantPar.Fa)/(plantPar.Ma+plantPar.Md);
theta2 = ones(1,tuningPar.simLength+1)*(plantPar.rhod-plantPar.rhoa)*plantPar.g/(plantPar.Ma+plantPar.Md);
theta3 = ones(1,tuningPar.simLength+1)*plantPar.betad;

figure();
subplot(211), plot(time, theta1, time, thetaHatS(1,:),'r--',time, thetaHatK(1,:),'g--');
xlabel('Time [s]');
ylabel('Value [?]');
legend('theta1','thetaHat(1)S','thetaHat(1)K');
grid on;

subplot(212), plot(time, theta2, time, thetaHatS(2,:),'r--', time, thetaHatK(2,:),'g--');
xlabel('Time [s]');
ylabel('Value [?]');
legend('theta2','thetaHat(2)S','thetaHat(2)K');
grid on;

% Error
% subplot(313), plot(time, theta3, time, theta3Hat,'r--');
% xlabel('Time [s]');
% ylabel('Value [?]');
% legend('theta3','theta3Hat');
% grid on;

%% Error plot
 figure()
% subplot(411), plot(time, Pc-PcHat);
% %axis([0 time(end) -1 1]);
% xlabel('Time [s]');
% ylabel('Error');
% legend('Pc-PcHat');
% grid on;
% 
subplot(311), plot(time, qbitK-qbitHatK, time, qbitK-qbitHatS);
%axis([0 time(end) -0.1 0.1]);
xlabel('Time [s]');
ylabel('Error');
legend('qbitErK', 'qbitErS');
grid on;

subplot(312), plot(time, theta1-thetaHatK(1,:),time, theta1-thetaHatS(1,:));
%axis([0 time(end) -10 10]); 
xlabel('Time [s]');
ylabel('Error');
legend('theta1ErK', 'theta1ErS');
grid on;

subplot(313), plot(time, theta2-thetaHatK(2,:),time, theta2-thetaHatS(2,:));
%axis([0 time(end) -1e-2 1e-2]);
xlabel('Time [s]');
ylabel('Error');
legend('theta2ErK', 'theta2ErS');
grid on;

%% ALT
figure();
subplot(711),plot(time, PpK,time, 10*PcK);
ylabel('Pressure [Bar]');
xlabel('Time [s]');
legend('Pp', '10*Pc', 'orientation', 'horizontal', 'location', 'North');
grid on;


subplot(712),plot(time, PbitK, time, PbitHatS, 'r--', time, PbitHatK, 'g--');
xlabel('Time [s]');
ylabel('Pressure [Bar]');
legend('Pbit', 'PbitHatS', 'PbitHatK', 'orientation', 'horizontal', 'location', 'North');
%axis([0 tuningPar.simLength 200 250]);
grid on;

subplot(713), plot(time, qpump*6e4, time, qback*6e4);
xlabel('Time [s]');
ylabel('Flow [l/min]');
legend('qpump', 'qback', 'orientation', 'horizontal', 'location', 'North');
grid on;

subplot(714), plot(time, qbitS*60e3, time, qbitHatS*60e3,'r--', time, qbitHatK*60e3,'g--');
xlabel('Time [s]');
ylabel('Flow [l/min]');
legend('qbit', 'qbitHatS', 'qbitHatK', 'orientation', 'horizontal', 'location', 'North');
grid on;

subplot(715), plot(time, qchokeS*60e3, time, 1000*Zc);
xlabel('Time [s]');
ylabel('Flow [l/min]');
legend('qchoke', '1000*Zc', 'orientation', 'horizontal', 'location', 'North');
grid on;


%The correct values...
theta1 = ones(1,tuningPar.simLength+1)*(plantPar.Fd+plantPar.Fa)/(plantPar.Ma+plantPar.Md);
theta2 = ones(1,tuningPar.simLength+1)*(plantPar.rhod-plantPar.rhoa)*plantPar.g/(plantPar.Ma+plantPar.Md);
theta3 = ones(1,tuningPar.simLength+1)*plantPar.betad;

subplot(716), plot(time, theta1, time, thetaHatS(1,:),'r--',time, thetaHatK(1,:),'g--');
xlabel('Time [s]');
ylabel('Value [?]');
legend('theta1','thetaHat(1)S','thetaHat(1)K', 'orientation', 'horizontal', 'location', 'North');
grid on;

subplot(717), plot(time, theta2, time, thetaHatS(2,:),'r--', time, thetaHatK(2,:),'g--');
xlabel('Time [s]');
ylabel('Value [?]');
legend('theta2','thetaHat(2)S','thetaHat(2)K', 'orientation', 'horizontal', 'location', 'North');
grid on;