%% Function to run simulation and make plots
clear all;
close all;
clc;

%Get constants
plantPar    = getPlantPar();
tuningPar   = getTuningPar();
initPar     = getInitPar(plantPar, tuningPar);
%[qpump qback]= getqpump(1000);

% load qpump2.mat;
% qpump = max(qpump, 100/6e4);
qpump(1:1000) = 1000/6e4;
qpump(1001:4000) = 500/6e4;
% qpump = getqpumpRamp();
% qpump = qpump+200/6e4;
qback = zeros(1,tuningPar.simLength+1);
%Run simulation
[Pp, Pc, qbit, qchoke, Pbit, thetaHat, qbitHat, PbitHat, pfiTheta]...
    = simulateGrip(plantPar, tuningPar, initPar, qpump, qback);

%PLOTS
time = 0:tuningPar.simLength;
M = plantPar.Ma + plantPar.Md;
g = plantPar.g;
rhod = plantPar.rhod;
rhoa = plantPar.rhoa;
Fd = plantPar.Fd;
Fa = plantPar.Fa;
F = Fd + Fa;

%% Pressures
figure();
subplot(211),plot(time, Pp,time, 10*Pc);
ylabel('Pressure [Bar]');
xlabel('Time [s]');
legend('Pp', '10*Pc');
grid on;


subplot(212),plot(time, Pbit);
xlabel('Time [s]');
ylabel('Pressure [Bar]');
legend('Pbit');
grid on;
%%
%Flows
figure();
subplot(311), plot(time, qpump(1:tuningPar.simLength+1)*6e4, time, qback*6e4);
xlabel('Time [s]');
ylabel('Flow [l/min]');
legend('qpump');%, 'qback');
grid on;

subplot(312), plot(time, qbit*60e3, time, qbitHat*6e4,'r--');
xlabel('Time [s]');
ylabel('Flow [l/min]');
legend('qbit', 'qbit (x(2))');
grid on;

subplot(313), plot(time, qchoke*60e3);
xlabel('Time [s]');
ylabel('Flow [l/min]');
legend('qchoke');
grid on;

%% Estimation

figure();
plot(time, qbit*6e4, time, qbitHat*6e4,'r--');
xlabel('Time [s]');
ylabel('Flow [ltr/min]');
legend('qbit','qbitHat');
grid on;
set(gcf, 'PaperUnits', 'centimeters');
set(gcf, 'PaperSize', [15 4]);
set(gcf, 'PaperPosition', [0 0 15 4]);
%pause
%saveas(gcf,'C:\Master\LaTeX\fig\stepTestGripqbit','pdf')

figure
plot(time, (Fa+Fd)/M*ones(1,tuningPar.simLength+1), time, thetaHat(1,:),'r--');
xlabel('Time [s]');
ylabel('[]');
legend('\theta_1','\theta_1 Hat');
grid on;
set(gcf, 'PaperUnits', 'centimeters');
set(gcf, 'PaperSize', [15 4]);
set(gcf, 'PaperPosition', [0 0 15 4]);
%pause
%saveas(gcf,'C:\Master\LaTeX\fig\stepTestGripTheta1','pdf')

figure
plot(time, (rhod-rhoa)*g/M*ones(1,tuningPar.simLength+1), time, thetaHat(2,:),'r--');
xlabel('Time [s]');
ylabel('[]');
legend('\theta_2','\theta_2 Hat');
grid on;
set(gcf, 'PaperUnits', 'centimeters');
set(gcf, 'PaperSize', [15 4]);
set(gcf, 'PaperPosition', [0 0 15 4]);
%pause
%saveas(gcf,'C:\Master\LaTeX\fig\stepTestGripTheta2','pdf')

figure
plot(time, Pbit, time, PbitHat,'r--');
xlabel('Time [s]');
ylabel('Pressure [Bar]');
legend('Pbit', 'PbitHat');
grid on;
axis([0 2000 225 300])
set(gcf, 'PaperUnits', 'centimeters');
set(gcf, 'PaperSize', [15 4]);
set(gcf, 'PaperPosition', [0 0 15 4]);
%pause
%saveas(gcf,'C:\Master\LaTeX\fig\stepTestGrippbit','pdf')

%% Error

figure();
subplot(311), plot(time, qbit-qbitHat);
xlabel('Time [s]');
ylabel('Value [?]');
legend('qbit-qbitHat');
grid on;


subplot(312), plot(time, (Fa+Fd)/M*ones(1,tuningPar.simLength+1)-(thetaHat(1,:)));
xlabel('Time [s]');
ylabel('Value [?]');
legend('F-FHat');
grid on;
subplot(313), plot(time, 0*ones(1,tuningPar.simLength+1)-(thetaHat(2,:)));
xlabel('Time [s]');
ylabel('Value [?]');
legend('rho-rhoHat');
grid on;

%% STates
figure();
subplot(311),plot(time, Pp,time, 10*Pc);
ylabel('Pressure [Bar]');
xlabel('Time [s]');
legend('Pp', '10*Pc');
grid on;

subplot(312),plot(time, Pbit);
xlabel('Time [s]');
ylabel('Pressure [Bar]');
legend('Pbit');
axis([0 tuningPar.simLength 200 250]);
grid on;

subplot(313), plot(time, qpump(1:tuningPar.simLength+1)*6e4, time, qback*ones(1,tuningPar.simLength+1)*6e4,time, qbit*60e3);
xlabel('Time [s]');
ylabel('Flow [l/min]');
legend('qpump', 'qback', 'qbit');
grid on;



