%% Function to run simulation and make plots
%clear all;
%close all;
clc;

%Get constants
plantPar    = getPlantPar();
tuningPar   = getTuningPar();
initPar     = getInitPar(plantPar, tuningPar);
%[qpump qback]= getqpump(tuningPar.simLength);

%load qpump2.mat;
%qpump = max(qpump, 100/6e4);
qpump(1:1000) = 1000/6e4;
qpump(1001:4000) = 500/6e4;
qback = 0;

%Run simulation
[Pp, Pc, qbit, qchoke, Pbit, qbitHat, PbitHat, thetaHat, PcHat]...
    = simulateKaasa(plantPar, tuningPar, initPar, qpump, qback, 1);
%[Pp2, Pc2, qbit2, qchoke2, Pbit2, qbitHat2, PbitHat2, thetaHat2, PcHat2]...
 %   = simulateKaasa(plantPar, tuningPar, initPar, qpump, qback, 20);
%[Pp3, Pc3, qbit3, qchoke3, Pbit3, qbitHat3, PbitHat3, thetaHat3, PcHat3]...
 %   = simulateKaasa(plantPar, tuningPar, initPar, qpump, qback, 50);

%PLOTS
time = 0:tuningPar.simLength;
M = plantPar.Ma + plantPar.Md;
g = plantPar.g;
rhod = plantPar.rhod;
rhoa = plantPar.rhoa;
Fd = plantPar.Fd;
Fa = plantPar.Fa;
F = Fd + Fa;

%% Pressures
figure();
subplot(211),plot(time, Pp,time, 10*Pc);
ylabel('Pressure [Bar]');
xlabel('Time [s]');
legend('Pp', '10*Pc');
grid on;


subplot(212),plot(time, Pbit);
xlabel('Time [s]');
ylabel('Pressure [Bar]');
legend('Pbit');
grid on;
%%
% Flows
% figure();
% subplot(311), plot(time, qpump*6e4);%, time, qback*6e4);
% xlabel('Time [s]');
% ylabel('Flow [l/min]');
% legend('qpump');%, 'qback');
% grid on;
% 
% subplot(312), plot(time, qbit*60e3, time, qbitHat*6e4,'r--');
% xlabel('Time [s]');
% ylabel('Flow [l/min]');
% legend('qbit', 'qbit (x(2))');
% grid on;
% 
% subplot(313), plot(time, qchoke*60e3);
% xlabel('Time [s]');
% ylabel('Flow [l/min]');
% legend('qchoke');
% grid on;

%% Estimation

figure();
plot(time, qbit*6e4, time, qbitHat*6e4,'--',...
    time, qbitHat2*6e4,'--',...
    time, qbitHat3*6e4,'--');
xlabel('Time [s]');
ylabel('Flow [ltr/min]');
legend('q_{bit}','q_{bit}Hat\tau = 1','q_{bit}Hat \tau = 20','q_{bit}Hat \tau = 50');
grid on;
set(gcf, 'PaperUnits', 'centimeters');
set(gcf, 'PaperSize', [15 4]);
set(gcf, 'PaperPosition', [0 0 15 4]);
%pause
%saveas(gcf,'C:\Master\LaTeX\fig\stepTestKaasaqbit','pdf')

%%
figure
plot(time, (Fa+Fd)/M*ones(1,tuningPar.simLength+1),...
    time, thetaHat(1,:),'r--');%,...
%    time, thetaHat2(1,:),'--',...
 %   time, thetaHat3(1,:),'--');
xlabel('Time [s]');
ylabel('[]');
legend('\theta_1','\theta_1Hat');% \tau=1','\theta_1Hat \tau=20','\theta_1Hat \tau=50');
grid on;
set(gcf, 'PaperUnits', 'centimeters');
set(gcf, 'PaperSize', [15 4]);
set(gcf, 'PaperPosition', [0 0 15 4]);
%pause
%saveas(gcf,'C:\Master\LaTeX\fig\stepTestKaasaTheta1','pdf')

figure
plot(time, (rhod-rhoa)*g/M*ones(1,tuningPar.simLength+1),...
    time, thetaHat(2,:),'r--');%,...
%    time, thetaHat2(2,:),'--',...
%    time, thetaHat3(2,:),'--');
xlabel('Time [s]');
ylabel('[]');
legend('\theta_2','\theta_2Hat');% \tau=1','\theta_2Hat \tau=20','\theta_2Hat \tau=50');
grid on;
set(gcf, 'PaperUnits', 'centimeters');
set(gcf, 'PaperSize', [15 4]);
set(gcf, 'PaperPosition', [0 0 15 4]);
%pause
%saveas(gcf,'C:\Master\LaTeX\fig\stepTestKaasaTheta2','pdf')
%%
figure
plot(time, Pbit,...
    time, PbitHat(1,:),'r--');%,...
%    time, PbitHat2(1,:),'--',...
%    time, PbitHat3(1,:),'--');
xlabel('Time [s]');
ylabel('Pressure [bar]');
legend('p_{bit}','p_{bit}Hat');% \tau=1','p_{bit}Hat \tau=20','p_{bit}Hat \tau=50');
grid on;
axis([0 2000 220 290])
set(gcf, 'PaperUnits', 'centimeters');
set(gcf, 'PaperSize', [15 4]);
set(gcf, 'PaperPosition', [0 0 15 4]);
%pause
%saveas(gcf,'C:\Master\LaTeX\fig\stepTestKaasapbit','pdf')



%% Error

figure();
subplot(311), plot(time, qbit-qbitHat);
xlabel('Time [s]');
ylabel('Value [?]');
legend('qbit-qbitHat');
grid on;


subplot(312), plot(time, (Fa+Fd)/M*ones(1,tuningPar.simLength+1)-(thetaHat(1,:)));
xlabel('Time [s]');
ylabel('Value [?]');
legend('F-FHat');
grid on;
subplot(313), plot(time, 0*ones(1,tuningPar.simLength+1)-(thetaHat(2,:)));
xlabel('Time [s]');
ylabel('Value [?]');
legend('rho-rhoHat');
grid on;