%% Function to run simulation and make plots
clear all;
close all;
clc;

%Get constants
plantPar    = getPlantPar();
tuningPar   = getTuningPar();
initPar     = getInitPar(plantPar, tuningPar);
%[qpump qback]= getqpump(tuningPar.simLength);
load qpump.mat;
qback = 0;

%Run simulation
[PpG, PcG, qbitG, qchokeG, PbitG, thetaHatG, qbitHatG, PbitHatG]...
    = simulateGrip(plantPar, tuningPar, initPar, qpump, qback);
[PpS, PcS, qbitS, qchokeS, PbitS, qbitHatS, PbitHatS, thetaHatS, PcHatS, theta3HatS]...
    = simulateStamnes(plantPar, tuningPar, initPar, qpump, qback);


%PLOTS
time = 0:tuningPar.simLength;
M = plantPar.Ma + plantPar.Md;
g = plantPar.g;
rhod = plantPar.rhod;
rhoa = plantPar.rhoa;
Fd = plantPar.Fd;
Fa = plantPar.Fa;

%% Pressures
figure();
subplot(211),plot(time, PpG, time, 10*PcG, time, PpS, time, 10*PcS);
ylabel('Pressure [Bar]');
xlabel('Time [s]');
legend('PpG', '10*PcG', 'PpS', '10*PcS');
grid on;


subplot(212),plot(time, PbitG,time, PbitS, time, PbitHatS, 'r--', time, PbitHatG, 'g--');
xlabel('Time [s]');
ylabel('Pressure [Bar]');
legend('PbitG', 'PbitS', 'PbitHatS', 'PbitHatG');
axis([0 tuningPar.simLength 200 250]);
grid on;

%% Flows
figure();
subplot(311), plot(time, qpump(mod(time,1001)+1)*6e4);%, time, qback*6e4);
xlabel('Time [s]');
ylabel('Flow [l/min]');
legend('qpump');
grid on;

subplot(312), plot(time, qbitS*60e3, time, qbitHatS*60e3,'r--', time, qbitHatG*60e3,'g--');
xlabel('Time [s]');
ylabel('Flow [l/min]');
legend('qbit', 'qbitHatS', 'qbitHatG');
grid on;

subplot(313), plot(time, qchokeS*60e3);%, time, 1000*Zc);
xlabel('Time [s]');
ylabel('Flow [l/min]');
legend('qchoke');
grid on;

%% Adaptation

figure();
subplot(211), plot(time, Fa*ones(1,tuningPar.simLength+1), time, M*thetaHatS(1,:)-Fd,'r--',time, M*thetaHatG(1,:)-Fd,'g--');
xlabel('Time [s]');
ylabel('Friction factor [-]');
legend('Fa','FaS','FaG');
grid on;

subplot(212), plot(time, rhoa*ones(1,tuningPar.simLength+1), time, rhod-M/g*thetaHatS(2,:),'r--', time, rhod-M/g*thetaHatG(2,:),'g--');
xlabel('Time [s]');
ylabel('Density [kg/m^3]');
legend('theta2','rhoaS','rhoaG');
grid on;

% subplot(313), plot(time, theta3, time, theta3Hat,'r--');
% xlabel('Time [s]');
% ylabel('Value [?]');
% legend('theta3','theta3Hat');
% grid on;

%% Error plot
  figure()
% % subplot(411), plot(time, Pc-PcHat);
% % %axis([0 time(end) -1 1]);
% % xlabel('Time [s]');
% % ylabel('Error');
% % legend('Pc-PcHat');
% % grid on;
% % 
subplot(311), plot(time, qbitG-qbitHatG, time, qbitG-qbitHatS);
%axis([0 time(end) -0.1 0.1]);
xlabel('Time [s]');
ylabel('Error');
legend('qbitErG', 'qbitErS');
grid on;

subplot(312), plot(time, (Fa+Fd)/(M)-thetaHatG(1,:),time, (Fa+Fd)/(M)-thetaHatS(1,:));
%axis([0 time(end) -10 10]); 
xlabel('Time [s]');
ylabel('Error');
legend('theta1ErG', 'theta1ErS');
grid on;

subplot(313), plot(time, (rhod-rhoa)*g/M-thetaHatG(2,:),time, (rhod-rhoa)*g/M-thetaHatS(2,:));
%axis([0 time(end) -1e-2 1e-2]);
xlabel('Time [s]');
ylabel('Error');
legend('theta2ErG', 'theta2ErS');
grid on;

% %% ALT
% figure();
% subplot(711),plot(time, PpG,time, 10*PcG);
% ylabel('Pressure [Bar]');
% xlabel('Time [s]');
% legend('Pp', '10*Pc', 'orientation', 'horizontal', 'location', 'North');
% grid on;
% 
% 
% subplot(712),plot(time, PbitG, time, PbitHatS, 'r--', time, PbitHatG, 'g--');
% xlabel('Time [s]');
% ylabel('Pressure [Bar]');
% legend('Pbit', 'PbitHatS', 'PbitHatG', 'orientation', 'horizontal', 'location', 'North');
% %axis([0 tuningPar.simLength 200 250]);
% grid on;
% 
% subplot(713), plot(time, qpump*6e4, time, qback*6e4);
% xlabel('Time [s]');
% ylabel('Flow [l/min]');
% legend('qpump', 'qback', 'orientation', 'horizontal', 'location', 'North');
% grid on;
% 
% subplot(714), plot(time, qbitS*60e3, time, qbitHatS*60e3,'r--', time, qbitHatG*60e3,'g--');
% xlabel('Time [s]');
% ylabel('Flow [l/min]');
% legend('qbit', 'qbitHatS', 'qbitHatG', 'orientation', 'horizontal', 'location', 'North');
% grid on;
% 
% subplot(715), plot(time, qchokeS*60e3, time, 1000*Zc);
% xlabel('Time [s]');
% ylabel('Flow [l/min]');
% legend('qchoke', '1000*Zc', 'orientation', 'horizontal', 'location', 'North');
% grid on;
% 
% 
% %The correct values...
% theta1 = ones(1,tuningPar.simLength+1)*(plantPar.Fd+plantPar.Fa)/(plantPar.Ma+plantPar.Md);
% theta2 = ones(1,tuningPar.simLength+1)*(plantPar.rhod-plantPar.rhoa)*plantPar.g/(plantPar.Ma+plantPar.Md);
% theta3 = ones(1,tuningPar.simLength+1)*plantPar.betad;
% 
% subplot(716), plot(time, theta1, time, thetaHatS(1,:),'r--',time, thetaHatG(1,:),'g--');
% xlabel('Time [s]');
% ylabel('Value [?]');
% legend('theta1','thetaHat(1)S','thetaHat(1)G', 'orientation', 'horizontal', 'location', 'North');
% grid on;
% 
% subplot(717), plot(time, theta2, time, thetaHatS(2,:),'r--', time, thetaHatG(2,:),'g--');
% xlabel('Time [s]');
% ylabel('Value [?]');
% legend('theta2','thetaHat(2)S','thetaHat(2)G', 'orientation', 'horizontal', 'location', 'North');
% grid on;