function [Pp, Pc, qbit, qchoke, Pbit, x, xHat, P, xDotUT] =...
    simulatePoly(plantPar, tuningPar, initPar, qpump, qback)


y0 = [initPar.Pp0 initPar.Pc0 initPar.qbit0 initPar.x0(1) initPar.x0(2) initPar.x0(3)... 
        initPar.xHat0(1) initPar.xHat0(2) initPar.xHat0(3) mat2vec(initPar.P0)']';

simLength = tuningPar.simLength;

Pp      = zeros(1, simLength+1);
Pc      = zeros(1, simLength+1);
Pbit    = zeros(1, simLength+1);
%PbitHat = zeros(1, simLength+1);
qbit    = zeros(1, simLength+1);
%qbitHat = zeros(1, simLength+1);
qchoke  = zeros(1, simLength+1);
xHat    = zeros(3, simLength+1);
x       = zeros(3, simLength+1);
P       = zeros(9, simLength+1);


Pp(1)       = initPar.Pp0;
Pc(1)       = initPar.Pc0;
qbit(1)     = initPar.qbit0;
%qbitHat(1)  = initPar.qbitHat0;
x(:,1)      = initPar.x0;
xHat(:,1)   = initPar.xHat0;
P(:,1)      = mat2vec(initPar.P0);


simLength   = tuningPar.simLength;
stepSize    = tuningPar.stepSize;


for time = 2:simLength+1
    clc
    time-1
    timeSpan = time:stepSize:time+1;
    
    [yOut xtraOut] = ode4m(@(t, y) totalSystem(t, y, plantPar, tuningPar, qpump(mod(time,4001)+1), qback), timeSpan, y0);
    
    Pp(time)        = yOut(end, 1);
    Pc(time)        = yOut(end, 2);
    qbit(time)      = yOut(end, 3);
    
    x(:,time)       = yOut(end,4:6);
    xHat(:,time)    = yOut(end,7:9);
    P(:,time)       = yOut(end, 10:end);
    
    
    qchoke(time)    = xtraOut(end).var1;
    Pbit(time)      = xtraOut(end).var2;
    xDotUT(time)   = xtraOut(end).var3;
   % qbitHat(time)   = xtraOut(end).var4;
   
    y0 = [yOut(end,1) yOut(end, 2) yOut(end,3) yOut(end,4) yOut(end,5) yOut(end,6) yOut(end,7) yOut(end,8) yOut(end,9) yOut(end,10:end)]';

end   
end

function [dydt xtra] = totalSystem(t,y,plantPar, tuningPar, qpump, qback)

%Plant parameters
Vd      = plantPar.Vd;
betad   = plantPar.betad;
Va      = plantPar.Va;
betaa   = plantPar.betaa;
Ma      = plantPar.Ma;
Md      = plantPar.Md;
M       = Ma+Md;
Fd      = plantPar.Fd;
Fa      = plantPar.Fa;
rhod    = plantPar.rhod;
rhoa    = plantPar.rhoa;
Kc      = plantPar.Kc;
P0      = plantPar.p0;
g       = plantPar.g;
hbit    = plantPar.hbit;



%Tuning parameters
qres    = tuningPar.qres;

H = tuningPar.H;
h1 = H(1,1);
h2 = H(2,2);
h3 = H(3,3);

B = tuningPar.B;
b = tuningPar.b;

A = [h1 0 0];


%Some defined constants
VaDot = 0;
a1 = betad/Vd;
a2 = 1/M;

%States
Pp      = y(1);
Pc      = y(2);
qbit    = y(3);
x       = y(4:6);
xHat    = y(7:9);
P       = vec2mat(y(10:end),3);

qback   = 0;
% Choke
Zc      = tuningPar.Zc;
Zc      = max(0,Zc);
Zc      = min(Zc, 1);
qchoke  = Kc*Zc*sqrt(2/rhoa*(Pc-P0));

% Plant
PpDot   = a1*(qpump - qbit);
PcDot   = betaa/Va*(qbit + qback -qchoke + qres - VaDot);
qbitDot = a2*(Pp - Pc -(Fa+Fd)*abs(qbit)*qbit + (rhod-rhoa)*g*hbit);

Pbit    = Pc + Ma*qbitDot + Fa*abs(qbit+qres)*(qbit+qres) + rhoa*g*hbit;

%System
f    = [a1/h1*(qpump - h2*x(2));
        a2/h2*(h1*x(1) - Pc - h3*h2^2*x(3)*abs(x(2))*x(2));
        0];
xDot = f + b*randn(3,1); 
y    = A*x + B*randn(1,1);

%Optimal estimation filter for polynomial systems'
F =[a1/h1*(qpump - h2*xHat(2));
    a2/h2*(h1*xHat(1) - Pc - h2^2*h3*(xHat(2)*P(3,2) + xHat(3)*P(2,2) + xHat(3)*xHat(2)*abs(xHat(2))));
    0];    
G =[0,          a2*h2*h3,                   0;
    -a1*h2/h1, -2*a2*h2*h3*xHat(2)*xHat(3), 0;
    0,       -a2*h2*h3*xHat(2)*abs(xHat(2)), 0];

xHatDot = F + P*A'*inv(B*B')*(y-A*xHat);

Pdot    = P*G + G'*P' + b*b' - P*A'*inv(B*B')*A*P;

%Additional output variables
xtra.var1 = qchoke;
xtra.var2 = Pbit;
xtra.var3 = xHatDot(1);
%xtra.var4 = qbitHat;
%xtra.var5 = thetaHat;


%Collect the dynamtics
dydt = [PpDot; PcDot; qbitDot; xDot(1); xDot(2); xDot(3); xHatDot(1); xHatDot(2); xHatDot(3); mat2vec(Pdot)];

end