function [Pp, Pc, qbit, qchoke, Pbit, qbitHat, PbitHat, thetaHat, PcHat, theta3Hat] =...
    simulateStamnes(plantPar, tuningPar, initPar, qpump, qback)


y0 = [initPar.Pp0 initPar.Pc0 initPar.qbit0 initPar.xiHat0 ...
    initPar.sigmaHat0(1) initPar.sigmaHat0(2) initPar.PcHat0 initPar.theta3Hat0]';

simLength = tuningPar.simLength;

Pp      = zeros(1, simLength+1);
Pc      = zeros(1, simLength+1);
Pbit    = zeros(1, simLength+1);
PbitHat = zeros(1, simLength+1);
qbit    = zeros(1, simLength+1);
qbitHat = zeros(1, simLength+1);
qchoke  = zeros(1, simLength+1);
thetaHat= zeros(2, simLength+1);
PcHat   = zeros(1, simLength+1);
theta3Hat=zeros(1, simLength+1);

Pp(1)       = initPar.Pp0;
Pc(1)       = initPar.Pc0;
qbit(1)     = initPar.qbit0;
qbitHat(1)  = initPar.qbitHat0;
thetaHat(:,1) = initPar.thetaHat0;
PcHat(1)    = initPar.PcHat0;
theta3Hat(1)= initPar.theta3Hat0;


simLength   = tuningPar.simLength;
stepSize    = tuningPar.stepSize;


for time = 2:simLength+1
    clc
    time-1
    timeSpan = time:stepSize:time+1;
    
    [yOut xtraOut] = ode4m(@(t, y) totalSystem(t, y, plantPar, tuningPar, qpump(mod(time,4001)+1), qback), timeSpan, y0);
    
    Pp(time)        = max(0,yOut(end, 1));
    Pc(time)        = max(plantPar.p0,yOut(end, 2));
    qbit(time)      = max(0,yOut(end, 3));
  % xiHat(time)     = yOut(end, 4);
  % sigmaHat(time)  = yOut(end, 5-6);
    PcHat(time)     = yOut(end,7);
    theta3Hat(time) = yOut(end,8);
    
    qchoke(time)    = xtraOut(end).var1;
    Pbit(time)      = xtraOut(end).var2;
    PbitHat(time)   = xtraOut(end).var3;
    qbitHat(time)   = xtraOut(end).var4;
    thetaHat(:,time)= xtraOut(end).var5;
   
    y0 = [yOut(end, 1) yOut(end, 2) yOut(end, 3) yOut(end,4) yOut(end,5) yOut(end,6) yOut(end,7) yOut(end,8)];
   
end   
end

function [dydt xtra] = totalSystem(t,y,plantPar, tuningPar, qpump, qback)
persistent eSum;
if(isempty(eSum))
    eSum = 0;
end

%Plant parameters
Vd      = plantPar.Vd;
betad   = plantPar.betad;
Va      = plantPar.Va;
betaa   = plantPar.betaa;
Ma      = plantPar.Ma;
Md      = plantPar.Md;
M       = Ma+Md;
Fd      = plantPar.Fd;
Fa      = plantPar.Fa;
rhod    = plantPar.rhod;
rhoa    = plantPar.rhoa;
Kc      = plantPar.Kc;
P0      = plantPar.p0;
g       = plantPar.g;
hbit    = plantPar.hbit;


%Tuning parameters
%Zc      = tuningPar.Zc;

%qpump   = tuningPar.qpump;
%qback   = tuningPar.qback;
qres    = tuningPar.qres;

l1      = tuningPar.l1;
l2      = tuningPar.l2;
Gamma   = tuningPar.Gamma;
gamma3  = tuningPar.gamma3;


% if t<300
%     qpump=1000/60000;%
%      qback=200/6e4;
% end

%Some defined constants
VaDot = 0;
a1 = betad/Vd;
a2 = 1/M;

%States
Pp      = max(0,y(1));
Pc      = max(P0,y(2));
qbit    = max(0,y(3));
xiHat   = y(4);
sigmaHat= [y(5);y(6)];
PcHat   = y(7);
theta3Hat=y(8);


% Choke
Zc      = tuningPar.Zc;
%PcRef   = tuningPar.PcRef;
%e       = Pc - PcRef;
%eSum    = eSum + e;
%Kp      = tuningPar.Kp;
%Ki      = tuningPar.Ki;
%Zc      = Kp*e + Ki*eSum;
Zc      = max(0,Zc);
Zc      = min(Zc, 1);
qchoke  = Kc*Zc*sqrt(2/rhoa*(Pc-P0));

% Plant
PpDot   = a1*(qpump - qbit);
PcDot   = betaa/Va*(qbit + qback -qchoke + qres - VaDot);
qbitDot = a2*(Pp - Pc -Fd*abs(qbit)*qbit -...
            Fa*abs(qbit + qres)*(qbit+qres) + (rhod - rhoa)*g*hbit);

Pbit    = Pc + Ma*qbitDot + Fa*abs(qbit+qres)*(qbit+qres) + rhoa*g*hbit;

%Observer
qbitHat = xiHat - l1*Pp;

eta         = Gamma*[abs(qbitHat)^3/(3*l1*a1); -hbit*qbitHat/(l1*a1)];
thetaHat    = sigmaHat - eta;


%PbitHat = Pc + Ma*(a2*(Pp-Pc)-thetaHat(1)*abs(qbitHat)*qbitHat + thetaHat(2)*hbit)...
 %           + (M*thetaHat(1)-Fd)*abs(qbitHat)*qbitHat + (rhod*g - M*thetaHat(2))*hbit;

%if (qbit > 0.0001)
    xiHatDot= -l1*a1*qbitHat-thetaHat(1)*abs(qbitHat)*qbitHat...
            + thetaHat(2)*hbit+a2*(Pp-Pc)+l1*a1*qpump;
        
    PbitHat = Pc + Ma*(a2*(Pp-Pc)-thetaHat(1)*abs(qbitHat)*qbitHat+thetaHat(2)*hbit)...
            + (M*thetaHat(1)-Fd)*abs(qbitHat)*qbitHat + (rhod*g - M*thetaHat(2))*hbit;
% else
%     xiHatDot = l1*a1*qpump;
%     PbitHat = Pc + (rhod*g - M*thetaHat(2))*hbit;
% end

        
%Theta3 observer
xi2Tld  = Pc - PcHat;
PcHatDot= theta3Hat*qbitHat/Va + theta3Hat*(qback-qchoke-VaDot)/Va + l2*xi2Tld;
        
%Adaptation

sigmaHatDot = Gamma*[-abs(qbitHat)*qbitHat; hbit]...
                *(-qbitHat+qpump) ...
                + Gamma*[(abs(qbitHat)*qbitHat)/(l1*a1); -hbit/(l1*a1)]*xiHatDot...
                + Gamma*[0; -qbitHat/(l1*a1)]*0;

%Parameter estimation
phi3        = (qbitHat+qback-qchoke-VaDot)/Va;
theta3HatDot= gamma3*phi3*xi2Tld;

% Additional output variables
xtra.var1 = qchoke;
xtra.var2 = Pbit;
xtra.var3 = PbitHat;
xtra.var4 = qbitHat;
xtra.var5 = thetaHat;


%Collect the dynamtics
dydt = [PpDot; PcDot; qbitDot; xiHatDot; sigmaHatDot(1); sigmaHatDot(2); PcHatDot; theta3HatDot];

end