function [Pp, Pc, qbit, qchoke, Pbit, qbitHat, PbitHat, thetaHat, PcHat, theta3Hat] = simulateSystem(plantPar, tuningPar)

persistent Pp0;
persistent Pc0;
persistent qbit0;
persistent xiHat0;
persistent sigmaHat0;
persistent qbitHat0;
persistent thetaHat0;
persistent PcHat0;
persistent theta3Hat0;

if(isempty(Pp0))
    Pp0 = 80;
end
if(isempty(Pc0))
    Pc0 = 2;
end
if(isempty(PcHat0))
    PcHat0 = Pc0;
end
if(isempty(qbit0))
    qbit0 = 0.016;
end
if(isempty(qbitHat0))
    qbitHat0 = 0.015;
end
if(isempty(xiHat0))
    l1 = tuningPar.l1;
    xiHat0 = qbitHat0+l1*Pp0;
end
if(isempty(thetaHat0))
    theta1 = (plantPar.Fd+plantPar.Fa)/(plantPar.Ma+plantPar.Md);
    theta2 = (plantPar.rhod-plantPar.rhoa)*plantPar.g/(plantPar.Ma+plantPar.Md);
    thetaHat0 = [1*theta1;theta2];%-0.0086];
end
if(isempty(sigmaHat0))
    l1 = tuningPar.l1;
    a1 = plantPar.betad/plantPar.Vd;
    hbit = plantPar.hbit;
    Gamma= tuningPar.Gamma;
    sigmaHat0 = thetaHat0 + Gamma*[abs(qbitHat0)^3/(3*l1*a1); -hbit*qbitHat0/(l1*a1)];
end
if(isempty(theta3Hat0))
    theta3Hat0 = 14000;
end

y0 = [Pp0 Pc0 qbit0 xiHat0 sigmaHat0(1) sigmaHat0(2) PcHat0 theta3Hat0]';

simLength = tuningPar.simLength;

Pp      = zeros(1, simLength+1);
Pc      = zeros(1, simLength+1);
Pbit    = zeros(1, simLength+1);
PbitHat = zeros(1, simLength+1);
qbit    = zeros(1, simLength+1);
qbitHat = zeros(1, simLength+1);
qchoke  = zeros(1, simLength+1);
thetaHat= zeros(2, simLength+1);
PcHat   = zeros(1, simLength+1);
theta3Hat=zeros(1, simLength+1);

Pp(1)       = Pp0;
Pc(1)       = Pc0;
qbit(1)     = qbit0;
%qchoke(1)   = 0;
%Pbit(1)     = 0;
qbitHat(1)  = qbitHat0;
%PbitHat(1)  = 0;
thetaHat(:,1) = thetaHat0;
PcHat(1)    = PcHat0;
theta3Hat(1)= theta3Hat0;

simLength = tuningPar.simLength;
stepSize  = tuningPar.stepSize;


for time = 2:simLength+1
    clc
    time
    timeSpan = time:stepSize:time+1;
    
    [yOut xtraOut] = ode4m(@(t, y) totalSystem(t, y, plantPar, tuningPar), timeSpan, y0);
    
    Pp(time)        = yOut(end, 1);
    Pc(time)        = max(plantPar.p0,yOut(end, 2));
    qbit(time)      = yOut(end, 3);
  %  xiHat(time)     = yOut(end, 4);
  %  sigmaHat(time)  = yOut(end, 5-6);
    PcHat(time)     = yOut(end,7);
    theta3Hat(time) = yOut(end,8);
    
    qchoke(time)    = xtraOut(end).var1;
    Pbit(time)      = xtraOut(end).var2;
    PbitHat(time)   = xtraOut(end).var3;
    qbitHat(time)   = xtraOut(end).var4;
    thetaHat(:,time)= xtraOut(end).var5;
    y0 = [yOut(end, 1) yOut(end, 2) yOut(end, 3) yOut(end,4) yOut(end,5) yOut(end,6) yOut(end,7) yOut(end,8)];
    %       Pp          Pc              qbit        xiHat             sigmaHat 
end   
end

function [dydt xtra] = totalSystem(t,y,plantPar, tuningPar)

%Plant parameters
Vd      = plantPar.Vd;
betad   = plantPar.betad;
Va      = plantPar.Va;
betaa   = plantPar.betaa;
Ma      = plantPar.Ma;
Md      = plantPar.Md;
M       = Ma+Md;
Fd      = plantPar.Fd;
Fa      = plantPar.Fa;
rhod    = plantPar.rhod;
rhoa    = plantPar.rhoa;
Kc      = plantPar.Kc;
P0      = plantPar.p0;
g       = plantPar.g;
hbit    = plantPar.hbit;

%Tuning parameters
Zc      = tuningPar.Zc;
qpump   = tuningPar.qpump;
qback   = tuningPar.qback;
qres    = tuningPar.qres;

l1      = tuningPar.l1;
l2      = tuningPar.l2;
Gamma   = tuningPar.Gamma;
gamma3  = tuningPar.gamma3;

%Some defined constants
VaDot = 0;
a1 = betad/Vd;
a2 = 1/M;

%States
Pp      = y(1);
Pc      = max(P0,y(2));
qbit    = y(3);
xiHat   = y(4);
sigmaHat= [y(5);y(6)];
PcHat   = y(7);
theta3Hat=y(8);


qchoke  = Kc*Zc*sqrt(2/rhoa*(Pc-P0));

% Plant
PpDot   = a1*(qpump - qbit);
PcDot   = betaa/Va*(qbit + qback -qchoke + qres - VaDot);
qbitDot = a2*(Pp - Pc -Fd*abs(qbit)*qbit -...
            Fa*abs(qbit + qres)*(qbit+qres) + (rhod - rhoa)*g*hbit);

Pbit    = Pc + Ma*qbitDot + Fa*abs(qbit + qres)*(qbit+qres) + rhoa*g*hbit;

%Observer
qbitHat = xiHat - l1*Pp;

eta         = Gamma*[abs(qbitHat)^3/(3*l1*a1); -hbit*qbitHat/(l1*a1)];
thetaHat    = sigmaHat - eta;


PbitHat = Pc + Ma*(a2*(Pp-Pc)-thetaHat(1)*abs(qbitHat)*qbitHat+thetaHat(2)*hbit)...
            + (M*thetaHat(1)-Fd)*abs(qbitHat)*qbitHat + (rhod*g - M*thetaHat(2))*hbit;

xiHatDot= -l1*a1*qbitHat-thetaHat(1)*abs(qbitHat)*qbitHat...
            + thetaHat(2)*hbit+a2*(Pp-Pc)+l1*a1*qpump;

    %Theta3 observer
xi2Tld  = Pc - PcHat;
PcHatDot= theta3Hat*qbitHat/Va + theta3Hat*(qback-qchoke-VaDot)/Va + l2*xi2Tld;
        
%Adaptation

sigmaHatDot = -l1*Gamma*[(abs(qbitHat)*qbitHat)/(l1*a1); -hbit/(l1*a1)]...
                *(-a1*qbitHat+a1*qpump) ...
                + Gamma*[(abs(qbitHat)*qbitHat)/(l1*a1); -hbit/(l1*a1)]*xiHatDot...
                + Gamma*[0; -qbitHat/(l1*a1)]*0;

%Parameter estimation
phi3        = (qbitHat+qback-qchoke-VaDot)/Va;
theta3HatDot= gamma3*phi3*xi2Tld;

% Additional output variables
xtra.var1 = qchoke;
xtra.var2 = Pbit;
xtra.var3 = PbitHat;
xtra.var4 = qbitHat;
xtra.var5 = thetaHat;


%Collect the dynamtics
dydt = [PpDot; PcDot; qbitDot; xiHatDot; sigmaHatDot(1); sigmaHatDot(2); PcHatDot; theta3HatDot];

end